#ifndef _CLASS_ARRAY_C_
#define _CLASS_ARRAY_C_

//
//  File Array.C
//
//  September 3 1992 - August 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Array.H"

// Array<T> : Dynamic size arrays holding pointers on T objects
//

template<class T>
  Array<T>::Array(const int size, const int chunk)
         : VArray(size, chunk)
{
  tab = new (T*[fAlloc*kChunk]);
  for(int i = 0; i < fAlloc*kChunk; i++)
    tab[i] = 0;
}

template<class T>
  Array<T>::Array(const Array<T>& a) 
          : VArray(a.fSize,a.kChunk) {
  tab = new (T*[fAlloc*kChunk]);
  for(int i = 0; i < fAlloc*kChunk; i++)
    tab[i] = a.tab[i];
}

template<class T>
  Array<T>::~Array() {
  delete[] tab;
}

template<class T>
  T*& Array<T>::operator[](const int i) {
    return tab[checkSize(i)];
  }

template<class T>
  T* Array<T>::get(const int i) const {
    if (i < size())
      return tab[i];
    else
      return 0;
  }

template<class T>
  void Array<T>::swap(const int i, const int j) {
  if (i == j)
    return;
  T* tmp = (*this)[i];
  (*this)[i] = (*this)[j];
  (*this)[j] = tmp;
}

template<class T>
  int Array<T>::empty() {
    return -resize(0);
}

template<class T>
  int Array<T>::crop() {
    int max = fSize - 1;

    while((max >= 0) && (tab[max] == 0))
      max --;

    return -resize(max + 1);
  }

template<class T>
  int Array<T>::compact() {
    int i;
    
    for(i = 0; i < fSize; i++) {
      if (tab[i] == 0) {
        int j = i+1;
        while((j < fSize) && (tab[j] == 0))
          j++;
        if (j == fSize)
          break;
        else {
          tab[i] = tab[j];
          tab[j] = 0;
        }
      }
    }
    return -resize(i);
  }

template<class T>
  int Array<T>::firstFree() const {
    int i;
    
    for(i = 0; i < fSize; i++) {
      if (tab[i] == 0)
        return i;
    }
    return i;
  }

template<class T>
  int Array<T>::resize(const int nsize) {
    if (nsize == fSize)
      return 0;
    int oldAlloc = fAlloc;
    calcAlloc(nsize);
    int csize = (nsize < fSize) ? nsize : fSize;
    if (fAlloc != oldAlloc) {
      T** ntab = new (T*[fAlloc*kChunk]);
      for(int i = 0; i < csize; i++)
        ntab[i] = tab[i];
      delete[] tab;
      tab = ntab;
    }
    for(int i = csize; i < (fAlloc*kChunk); i++)
      tab[i] = 0;
    fSize = nsize;
    return (fAlloc - oldAlloc)*kChunk;
  }

#endif
