#ifndef _CLASS_ARRAY_H_
#define _CLASS_ARRAY_H_

//
//  File Array.H
//
//  September 3 1992 - August 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "common.H"

// Class VArray is an abstract base class for arrays of any type
// of elements and of dynamic size.
//

class VArray {
  public:
    enum {DefaultChunk = 5};
    VArray(const int size, const int chunk);
    VArray(const VArray& v);
    virtual int size() const;
    virtual void swap(const int i, const int j) = 0;
  protected:
    const int kChunk;
    int fAlloc;
    int fSize;
    virtual int resize(const int nsize) = 0;
    virtual int checkSize(const int i);
    virtual void calcAlloc(const int maxIdx);
};

// Class Array<T> implements arrays of pointers on type T.
// The size of the array grows dynamically as elements are indexed.
// The size of the array is the highest index ever adressed + 1.
// The size of the array can be reduced in three ways:
//   - the empty() method empties the array (so size = 0 and minimum memory)
//   - the crop() method suppresses all trailing null elements.
//   - the compact() method suppresses all null elements and fills the holes
//     by shifting the next non null element to the left.
//

template<class T> class Array : public VArray {
  public:
    Array(const int size = 0, const int chunk = DefaultChunk);
    Array(const Array<T>& a);
    virtual ~Array();
    virtual T*& operator[](const int i);
    virtual T* get(const int i) const;
    virtual void swap(const int i, const int j);
    virtual int empty();
    virtual int crop();
    virtual int compact();
    virtual int firstFree() const;
  protected:
    T** tab;
    virtual int resize(const int nsize);
};

#ifdef __GNUG__
#include "Array.C"
#endif

#endif
