#ifndef _CLASS_CONSTANT_C_
#define _CLASS_CONSTANT_C_

//
//  File Constant.C
//
//  September 3 1992 - September 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Constant.H"

template<class sigType>
  Constant<sigType>::Constant(const sigType& value, const bool always,
                              const char* name, const bool sched)
                   : Synchronous(name, sched),
                     cfVal(value),
                     fOut(this,"out"),
                     fShouldEmit(true),
                     cfAlways(always)
{}

template<class sigType>
  const InputSignal** Constant<sigType>::inSigList() const {
    const InputSignal** sigList;
    sigList = (const InputSignal**) new (InputSignal*[1]);
    sigList[0] = 0;
    return sigList;
  }

template<class sigType>
  const OutputSignal** Constant<sigType>::outSigList() const {
    const OutputSignal** sigList;
    sigList = (const OutputSignal**) new (OutputSignal*[2]);
    sigList[0] = &fOut;
    sigList[1] = 0;
    return sigList;
  }

template<class sigType>
  OutSignal<sigType>& Constant<sigType>::operator()() {
    return fOut;
}

template<class sigType>
  OutSignal<sigType>& Constant<sigType>::output() {
    return fOut;
}

template<class sigType>
  void Constant<sigType>::resetOutputs() {
    fOut.value().reset();
}

template<class sigType>
  void Constant<sigType>::setInputs() {
}

template<class sigType>
  void Constant<sigType>::resetInputs() {
}

template<class sigType>
  void Constant<sigType>::activate() {
    if(fShouldEmit) {
      fOut.value() = cfVal;
    fShouldEmit = cfAlways;
  }
}

template<class sigType>
  void Constant<sigType>::backup() {
    pf_backup_copy_ = fShouldEmit;
  }

template<class sigType>
  void Constant<sigType>::restore() {
    fShouldEmit = pf_backup_copy_;
  }

#endif

