#ifndef _CLASS_CONSTANT_H_
#define _CLASS_CONSTANT_H_

//
//  File Constant.H
//
//  September 3 1992 - September 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Synchronous.H"
#include "Signal.H"

// Instances of class Constant<sigType> yield a constant
// signal of type sigType.
// The value of the signal is given at instanciation time,
// and the signal can be emmitted once or at every tick 
// according to the value of the always parameter.

class PureConstant : virtual public Synchronous {
  public:
    PureConstant(const PureSample& value, const bool always = true,
             const char* name = "PureConstant", const bool sched = true);
    PureOutSignal& operator()();
    PureOutSignal& output();
    const InputSignal** inSigList() const;
    const OutputSignal** outSigList() const;
    void resetOutputs();
    void resetInputs();
    void setInputs();
    void activate();
    virtual void backup();
    virtual void restore();
  private:
    const PureSample cfVal;
    PureOutSignal fOut;
    bool fShouldEmit;
    const bool cfAlways;
    bool pf_backup_copy_;
    PureConstant(const PureConstant&);
};

template<class sigType>
  class Constant : virtual public Synchronous {
  public:
    Constant(const sigType& value, const bool always = true,
             const char* name = "Constant", const bool sched = true);
    OutSignal<sigType>& operator()();
    OutSignal<sigType>& output();
    const InputSignal** inSigList() const;
    const OutputSignal** outSigList() const;
    void resetOutputs();
    void resetInputs();
    void setInputs();
    void activate();
    virtual void backup();
    virtual void restore();
  private:
    const sigType cfVal;
    OutSignal<sigType> fOut;
    bool fShouldEmit;
    const bool cfAlways;
    bool pf_backup_copy_;
    Constant(const Constant<sigType>&);
};

#ifdef __GNUG__
#include "Constant.C"
#endif

#endif

