#ifndef _CLASS_DELAY_C_
#define _CLASS_DELAY_C_

//
//  File Delay.C
//
//  September 3 1992 - September 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Delay.H"
#include "Signal.H"
#include "Sample.H"

template<class sigType>
  Delay<sigType>::Delay(const sigType* initValue, const char* name,
                        const bool schedule)
                 : Synchronous(name, schedule),
                   fInput(this, "Input"),
                   fOutput(this, "Output", false), pf_backup_copy_(0) {
    if(initValue != 0)        // If there is an initial value
      lastValue = *initValue; // store it as last input value
  }

template<class sigType>
  Delay<sigType>::~Delay() {
    if (pf_backup_copy_ != 0)
      delete pf_backup_copy_;
  }

template<class sigType>
  const InputSignal** Delay<sigType>::inSigList() const {
    const InputSignal** inList;
  
    inList = (const InputSignal**) new (InputSignal*[2]);
    inList[0] = &fInput;
    inList[1] = 0;
    return inList;
  }

template<class sigType>
  const OutputSignal** Delay<sigType>::outSigList() const {
    const OutputSignal** sigList;
  
    sigList = (const OutputSignal**) new (OutputSignal*[2]);
    sigList[0] = &fOutput;
    sigList[1] = 0;
    return sigList;
  }

template<class sigType>
  InSignal<sigType>& Delay<sigType>::input() {
    return fInput;
  }

template<class sigType>
  OutSignal<sigType>& Delay<sigType>::output() {
    return fOutput;
  }

template<class sigType>
  void Delay<sigType>::resetOutputs() {  // When nobody needs our output
    fOutput.value() = lastValue;         // any longer, we can change it.
  }

template<class sigType>
  void Delay<sigType>::setInputs() {
    fInput.update();
  }

template<class sigType>
  void Delay<sigType>::resetInputs() {
    fInput.value().reset();
  }

template<class sigType>
  void Delay<sigType>::activate() {    // Just store input value
    if (fInput.value().isPresent())
      lastValue = fInput.value();      // in delay buffer
    else
      lastValue.reset();
  }

template<class sigType>
  void Delay<sigType>::backup() {
    if (pf_backup_copy_ == 0) {
      pf_backup_copy_ = new Sample<sigType>(lastValue);
    } else {
      *pf_backup_copy_ = lastValue;
    }
  }

template<class sigType>
  void Delay<sigType>::restore() {
    if (pf_backup_copy_ != 0) {
      lastValue = *pf_backup_copy_;
    }
}

//template<class sigType>
//  Delay<sigType>& Delay<sigType>::operator=(const Delay<sigType>& m) {
//  fInput.value() = ((Delay<sigType>&)m).fInput.value();
//  fOutput.value() = ((Delay<sigType>&)m).fOutput.value();
//  lastValue = m.lastValue;
//  return *this;
//}

#endif
// End of file Delay.C

