#ifndef _CLASS_DELAY_H_
#define _CLASS_DELAY_H_

//
//  File Delay.H
//
//  September 3 1992 - September 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "common.H"
#include "Synchronous.H"
#include "Signal.H"

#include "Sample.H"

//
// Delays delays a signal by one tick of the clock.
// An initial value can be specified for the first sample.
//

template<class sigType>
  class Delay : virtual public Synchronous {
  public:
    Delay(const sigType* initValue = 0, const char* name = "Delay",
          const bool schedule = true);
    ~Delay();
    const InputSignal** inSigList() const;
    const OutputSignal** outSigList() const;
    virtual InSignal<sigType>& input();
    virtual OutSignal<sigType>& output();
    void resetOutputs();
    void resetInputs();
    void setInputs();
    void activate();
    virtual void backup();
    virtual void restore();
  private:
    InSignal<sigType> fInput;
    OutSignal<sigType> fOutput;
    Sample<sigType> lastValue;    // Delay buffer for one tick
    Sample<sigType> *pf_backup_copy_;
    Delay(const Delay<sigType>&);
};

class PureDelay : virtual public Synchronous {
  public:
    PureDelay(const SBoolean* initValue = 0, const char* name = "PureDelay",
              const bool schedule = true);
    PureDelay(const PureSample& initValue, const char* name = "PureDelay",
              const bool schedule = true);
    ~PureDelay();
    const InputSignal** inSigList() const;
    const OutputSignal** outSigList() const;
    virtual PureInSignal& input();
    virtual PureOutSignal& output();
    void resetOutputs();
    void resetInputs();
    void setInputs();
    void activate();
    virtual void backup();
    virtual void restore();
  private:
    PureInSignal fInput;
    PureOutSignal fOutput;
    PureSample lastValue;    // Delay buffer for one tick
    PureSample *pf_backup_copy_;
    PureDelay(const PureDelay&);
};

#ifdef __GNUG__
#include "Delay.C"
#endif

#endif

// End of file Delay.H

