//
//  File Esterel.C
//
//  September 3 1992 - September 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Esterel.H"

const Esterel::tActionNumber Esterel::kGotoAction = 0;

Esterel::Esterel(const tStateNumber *const *const s,
                 const tStateNumber *const *const d)
        : Synchronous(""), cStateTable(s), cDagTable(d), pf_backup_copy_(0) {
}

void Esterel::branchIfFalse(const bool c,tActionNumber *i) {
  if (c)
    (*i)++;
  else
    (*i) += fActTable[*i];
}

void Esterel::gotoState(tStateNumber i) {
	fState = i;
	fActTable = cStateTable[i];
}

void Esterel::executeOpenDAG(tActionNumber *i) {
  const tActionNumber *oldTab = fActTable;
  fActTable = cDagTable[fActTable[*i]];
  tActionNumber j = 0;
  tActionNumber op;
  while((op = fActTable[j++]) != kGotoAction) {
  	doAction(op,&j);
  }
  fActTable = oldTab;
  (*i)++;
}

void Esterel::gotoClosedDAG(tActionNumber *i) {
	fActTable = cDagTable[fActTable[*i]];
	*i = 0;
}

void Esterel::activate() {
  tActionNumber op;
  tStateNumber i = 0;

  while( (op = fActTable[i++]) != kGotoAction ) {
    doAction(op, &i);
  }
  gotoState(fActTable[i]);
}

void Esterel::backup() {
	if (pf_backup_copy_ == 0) {
		pf_backup_copy_ = new EsterelState;
	}
	pf_backup_copy_->fActTable = fActTable;
	pf_backup_copy_->fState = fState;
}

void Esterel::restore() {
  if (pf_backup_copy_ != 0) {
  	fActTable = pf_backup_copy_->fActTable;
  	fState = pf_backup_copy_->fState;
  }
}

SBoolean Esterel::EvalBoth_OR(const SBoolean& a, const SBoolean& b) const {
  return a?(b?true:true):(b?true:false);  // Force evaluation of both args
}

SBoolean Esterel::EvalBoth_AND(const SBoolean& a, const SBoolean& b) const {
  return a?(b?true:false):(b?false:false);  // Force evaluation of both args
}

// End of file Esterel.C
