#ifndef _CLASS_ESTEREL_H_
#define _CLASS_ESTEREL_H_

//
//  File Esterel.H
//
//  September 3 1992 - September 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "common.H"
#include "Synchronous.H"
#include "SString.H"
#include "SBoolean.H"


typedef int integer;    // Esterel integer type definition.
typedef SString string;  // Esterel string type definition.
typedef SBoolean boolean;  // Esterel boolean type definition.

class Esterel : virtual public Synchronous {
  public:
    typedef unsigned long tStateNumber;
    typedef unsigned long tActionNumber;
    static const tActionNumber kGotoAction;

  protected:
    const tActionNumber *const *const cStateTable;  // Table of states
    const tActionNumber *const *const cDagTable;    // Table of dags
    
    const tActionNumber *fActTable;      // Current action table
    tStateNumber fState;                 // Current state
    
    virtual void doAction(tActionNumber act, tActionNumber *i) = 0;
    void branchIfFalse(const bool c, tActionNumber *i);
    void gotoState(tStateNumber i);
    void executeOpenDAG(tActionNumber *i);
    void gotoClosedDAG(tActionNumber *i);

  public:
    Esterel(const tStateNumber *const *const s,
            const tStateNumber *const *const d = 0);
    virtual void activate();  // Do the real job
    virtual void backup();
    virtual void restore();
            // Logical functions that evaluate their two arguments
    virtual SBoolean EvalBoth_OR(const SBoolean& a, const SBoolean& b) const;
    virtual SBoolean EvalBoth_AND(const SBoolean& a, const SBoolean& b) const;
  private:
    Esterel(const Esterel&);
    class EsterelState {
    	public:
   	    const tActionNumber * fActTable;
   	    tStateNumber fState;
   	} *pf_backup_copy_;
};

#endif

// End of file Esterel.H
