#ifndef _CLASS_IOMODULE_H_
#define _CLASS_IOMODULE_H_

//
//  File IOModule.H
//
//  September 3 1992 - November 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "common.H"
#include "Synchronous.H"

#include "Signal.H"

//
// InputModules allow input of signal through cin
//

class PureInModule : virtual public Synchronous {
    public:
      PureInModule(const char *prompt = "> ", const char *tail = "",
                   const bool errNotPres = false,
                   const char* name = "InputModule",
                   const bool schedule = true);
      virtual void setStreams(istream& i, ostream& o = cout, ostream& l = nullDev);
      const InputSignal** inSigList() const;
      const OutputSignal** outSigList() const;
      virtual PureOutSignal& input();
      virtual PureOutSignal& operator()();
      void resetOutputs();
      void resetInputs();
      void setInputs();
      void activate();
      virtual void backup();
      virtual void restore();
    private:
      istream* fInputStream;
      ostream* fOutputStream;
      ostream* fLogStream;
      const char *fPrompt;
      const char *fTail;
      PureOutSignal fInput;
      const bool ignoreErrs;
      bool fRestoring;
      bool fBackuping;
      PureOutSignal *pf_backup_copy_;
      PureInModule(const PureInModule&);
};

template<class sigType>
  class InputModule : virtual public Synchronous {
    public:
      InputModule(const char *prompt = "> ", const char *tail = "",
            const bool errNotPres = false,
                  const char* name = "InputModule",
                  const bool schedule = true);
      virtual void setStreams(istream& i, ostream& o = cout, ostream& l = nullDev);
      const InputSignal** inSigList() const;
      const OutputSignal** outSigList() const;
      virtual OutSignal<sigType>& input();
      virtual OutSignal<sigType>& operator()();
      void resetOutputs();
      void resetInputs();
      void setInputs();
      void activate();
      virtual void backup();
      virtual void restore();
    private:
      istream* fInputStream;
      ostream* fOutputStream;
      ostream* fLogStream;
      const char *fPrompt;
      const char *fTail;
      OutSignal<sigType> fInput;
      const bool ignoreErrs;
      bool fRestoring;
      bool fBackuping;
      OutSignal<sigType> *pf_backup_copy_;
      InputModule(const InputModule<sigType>&);
  };

//
// OutputModules allow writing a signal s value to cout
//

class PureOutModule : virtual public Synchronous {
    public:
      PureOutModule(const char *banner = "-> ", const bool always = true,
                    const char *tail = "\n", const char* name = "OutputModule",
                    const bool schedule = true);
      virtual void setStreams(ostream& o, ostream& l = nullDev);
      const InputSignal** inSigList() const;
      const OutputSignal** outSigList() const;
      virtual PureInSignal& probe();
      virtual PureInSignal& operator()();
      void resetOutputs();
      void resetInputs();
      void setInputs();
      void activate();
      virtual void backup();
      virtual void restore();
    private:
      ostream* fOutputStream;
      ostream* fLogStream;
      const char *fBanner;
      const char *fTail;
      bool fAlways;
      PureInSignal fProbe;
      PureOutModule(const PureOutModule&);
};

template<class sigType>
  class OutputModule : virtual public Synchronous {
    public:
      OutputModule(const char *banner = "-> ", const bool always = true,
                   const char *tail = "\n", const char* name = "OutputModule",
                   const bool schedule = true);
      virtual void setStreams(ostream& o, ostream& l = nullDev);
      const InputSignal** inSigList() const;
      const OutputSignal** outSigList() const;
      virtual InSignal<sigType>& probe();
      virtual InSignal<sigType>& operator()();
      void resetOutputs();
      void resetInputs();
      void setInputs();
      void activate();
      virtual void backup();
      virtual void restore();
    private:
      ostream* fOutputStream;
      ostream* fLogStream;
      const char *fBanner;
      const char *fTail;
      bool fAlways;
      InSignal<sigType> fProbe;
      OutputModule(const OutputModule<sigType>&);
};

#ifdef __GNUG__
#include "IOModule.C"
#endif

#endif

// End of file IOModule.H

