#ifndef _CLASS_PROTOSIM_C_
#define _CLASS_PROTOSIM_C_

//
//  File ProtoSim.C
//
//  September 1994 - October 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "ProtoSim.H"

template<class sigType>
	ProtoSim& plug(ProtoSim& x,InSignal<sigType>& s, const char *typ) {
		ostrstream buf;
		buf << s.name() << ends;
		InputModule<sigType> *i = new InputModule<sigType>("","",true,buf.str(),false);
		strstream *t = new strstream;
	
		x.fInModules[x.fNbInputs] = i;
		x.fInStreams[x.fNbInputs] = t;
		i->setStreams(*t, nullDev, nullDev);
		x.fInNames[x.fNbInputs] = buf.str();
		x.fInPure[x.fNbInputs] = new int(ProtoSim::eValued);
		x.fInTypes[x.fNbInputs] = (char*)typ;
		s << (*i)();
		x.fNbInputs++;
		return x;
	}

template<class sigType>
	ProtoSim& plug(ProtoSim& x,InSignal<sigType>& s, const char *typ, const char *nickname) {
		ostrstream buf;
		buf << nickname << ends;
		InputModule<sigType> *i = new InputModule<sigType>("","",true,buf.str(),false);
		strstream *t = new strstream;
	
		x.fInModules[x.fNbInputs] = i;
		x.fInStreams[x.fNbInputs] = t;
		i->setStreams(*t, nullDev, nullDev);
		x.fInNames[x.fNbInputs] = buf.str();
		x.fInPure[x.fNbInputs] = new int(ProtoSim::eValued);
		x.fInTypes[x.fNbInputs] = (char*)typ;
		s << (*i)();
		x.fNbInputs++;
		return x;
	}

template<class sigType>
	ProtoSim& plug(ProtoSim& x,OutSignal<sigType>& s, const char *typ) {
		ostrstream buf;
		buf << s.name() << ends;
		OutputModule<sigType> *o = new OutputModule<sigType>("",true,"",buf.str(),false);
		x.fClock->manage(o);
		strstream *t = new strstream;
	
		x.fOutStreams[x.fNbOutputs] = t;
		o->setStreams(*t, nullDev);
		x.fOutNames[x.fNbOutputs] = buf.str();
		x.fOutPure[x.fNbOutputs] = new int(ProtoSim::eValued);
		x.fOutTypes[x.fNbOutputs] = (char*)typ;
		(*o)() << s;
		x.fNbOutputs++;
		return x;
	}

template<class sigType>
	ProtoSim& plug(ProtoSim& x,OutSignal<sigType>& s, const char *typ, const char *nickname) {
		ostrstream buf;
		buf << nickname << ends;
		OutputModule<sigType> *o = new OutputModule<sigType>("",true,"",buf.str(),false);
		x.fClock->manage(o);
		strstream *t = new strstream;
	
		x.fOutStreams[x.fNbOutputs] = t;
		o->setStreams(*t, nullDev);
		x.fOutNames[x.fNbOutputs] = buf.str();
		x.fOutPure[x.fNbOutputs] = new int(ProtoSim::eValued);
		x.fOutTypes[x.fNbOutputs] = (char*)typ;
		(*o)() << s;
		x.fNbOutputs++;
		return x;
	}

#ifdef __GNUG__
#define _GNUG_DECLARE_PROTOPLUG_(T) \
ProtoSim& plug(ProtoSim& x, InSignal<##T##>& s, const char *typ); \
ProtoSim& plug(ProtoSim& x, InSignal<##T##>& s, const char *typ, const char *nickname); \
ProtoSim& plug(ProtoSim& x, OutSignal<##T##>& s, const char *typ); \
ProtoSim& plug(ProtoSim& x, OutSignal<##T##>& s, const char *typ, const char *nickname);
#endif

#endif
