#ifndef _CLASS_PROTOSIM_H_
#define _CLASS_PROTOSIM_H_

//
//  File ProtoSim.H
//
//  September 1994 - October 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)


#include "Signal.H"
#include "IOModule.H"
#include "Array.H"
#include "SScheduler.H"

#include <strstream.h>

class ProtoSim {
	public:
		enum  { eValued, ePure };
		enum  { eConcise, eVerbose };

		ProtoSim(const char *name, SScheduler *clock = 0);
		~ProtoSim();
		static void runsimul();

		void declare();
		void doTick();
		void doReset();
		void doInput(const char* name, const char* value);

// The following members should be private, but g++ and cxx
// do not accept templates of friends, and even CC produces
// wrong C code when more than two template friends are
// declared with the same anonymous type!
		int fNbInputs;
		int fNbOutputs;
		SScheduler *fClock;
		Array<strstream> fInStreams;
		Array<strstream> fOutStreams;
		Array<char> fInNames;
		Array<int> fInPure;
		Array<char> fInTypes;
		Array<char> fOutNames;
		Array<int> fOutPure;
		Array<char> fOutTypes;
		Array<Synchronous> fInModules;

	private:
		const char *fName;
		const int fInstNum;
		
		static const char *fVersion;
		
		static Array<ProtoSim> fInstances;
		static bool fMustQuit;
		static bool fVerbose;
		static int fMode;
		
		static void tick(istrstream* line);
		static void input(istrstream* line);
		static void reset(istrstream* line);
		
		static ProtoSim* lookUpInst(const char* name);

		int lookUpInput(const char* name);
		ProtoSim(const ProtoSim&);
};

ProtoSim& plug(ProtoSim& x, PureInSignal& s);
ProtoSim& plug(ProtoSim& x, PureInSignal& s, const char *nickname);
ProtoSim& plug(ProtoSim& x, PureOutSignal& s);
ProtoSim& plug(ProtoSim& x, PureOutSignal& s, const char *nickname);

template<class sigType>
	ProtoSim& plug(ProtoSim& x, InSignal<sigType>& s, const char *typ);
template<class sigType>
	ProtoSim& plug(ProtoSim& x, InSignal<sigType>& s, const char *typ, const char *nickname);
template<class sigType>
	ProtoSim& plug(ProtoSim& x, OutSignal<sigType>& s, const char *typ);
template<class sigType>
	ProtoSim& plug(ProtoSim& x, OutSignal<sigType>& s, const char *typ, const char *nickname);

#ifdef __GNUG__
#include "ProtoSim.C"
#endif

#endif
