//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#include <string.h>
#include <strstream.h>
#include "SString.H"

SString::SString(const char* s)
{
  ostrstream tmp;
  tmp << s << ends;
  buf = tmp.str();
}

SString::SString(const SString& s)
{
  ostrstream tmp;
  tmp << (char*)s << ends;
  buf = tmp.str();
}

SString::SString() {
  buf = 0;
}

SString::~SString() {
  if (buf != 0)
    delete[] buf;
}

SString::operator char*() const{
  ostrstream tmp;
  tmp << buf << ends;
  return tmp.str();
}

int SString::length() const {
  return strlen(buf);
}

char& SString::operator[](const int i) {
  return buf[i];
}

int SString::operator>(const SString& s) const {
  return (strcmp(buf, s.buf) > 0);
}

int SString::operator<(const SString& s) const {
  return (strcmp(buf, s.buf) < 0);
}

int SString::operator==(const SString& s) const {
  return (strcmp(buf, s.buf) == 0);
}

int SString::operator!=(const SString& s) const {
  return (strcmp(buf, s.buf) != 0);
}

int SString::operator>=(const SString& s) const {
  return (strcmp(buf, s.buf) >= 0);
}

int SString::operator<=(const SString& s) const {
  return (strcmp(buf, s.buf) <= 0);
}

SString& SString::operator=(const SString& s) {
  ostrstream tmp;
  tmp << s.buf << ends;
  if (buf != 0)
    delete[] buf;
  buf = tmp.str();
  return (*this);
}

SString& SString::operator<<(const SString& s) {
  ostrstream tmp;
  if (buf != 0)
    tmp << buf;
  if (s.buf != 0)
    tmp << s.buf;
  tmp << ends;
  if (buf != 0)
    delete[] buf;
  buf = tmp.str();
  return (*this);
}

ostream& operator<<(ostream& o, const SString& s) {
  if (s.buf != 0)
    o << s.buf;
  return o;
}

istream& operator>>(istream& i, SString& s) {
  strstreambuf tmp;
  i.get(tmp);
  tmp.sputc(0);
  if (s.buf != 0)
    delete[] s.buf;
  s.buf = tmp.str();
  return i;
}

