#ifndef _CLASS_SAMPLE_C_
#define _CLASS_SAMPLE_C_

//
//  File  Sample.C
//
//  September 3 1992 - May 1995
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include  "Sample.H"

template<class  sigType>
  Sample<sigType>::Sample() {
    fPresent  = false;
  }

template<class  sigType>
  Sample<sigType>::Sample(const sigType& val) {
    fPresent  = false;
    fValue = val;
  }

template<class  sigType>
  Sample<sigType>::Sample(const Sample<sigType>& s) {
    fPresent  = s.fPresent;
    fValue = s.fValue;
  }

template<class  sigType>
  bool Sample<sigType>::isPresent()  const {
    return  bool(fPresent);
  }

template<class  sigType>
  Sample<sigType>& Sample<sigType>::operator=(const sigType& v) {
    fValue  = v;
    fPresent  = true;
    return  *this;
  }

template<class  sigType>
  Sample<sigType>& Sample<sigType>::operator=(const Sample<sigType>& s) {
    fValue  = s.fValue;
    fPresent  = s.fPresent;
    return  *this;
  }

template<class sigType>
  const sigType& Sample<sigType>::value() const {
    return fValue;
  }

template<class sigType>
  void Sample<sigType>::reset() {
    fPresent = false;
  }


//  Sample input and output
//
template<class sigType>
  istream& operator>>(istream& i, Sample<sigType>& s) {
    sigType tmp;
    int c;
    bool fake = false;

    if (!i.good())
      return i;

    if ((i.peek()) == '\n')     // If empty line -> no data
      return i;
      
    if (i.flags() & ios::skipws)  // If should skip white spaces
      i >> ws;

    if (((c = i.get()) == '#') && (i.peek() == '(')) {
      i.get();           // If fake data (output of an OutputModule
      fake = true;       // when a signal is not present).
    } else
      i.putback(c);
      
    i >> tmp;
    if (i.good() && !fake){
      s.fValue = tmp;
      s.fPresent = true;
    }
    if (fake) {
      if (i.flags() & ios::skipws)  // If should skip white spaces
        i >> ws;
      if (i.peek() == ')')
        i.get();
      else
        i.clear(i.rdstate() | ios::badbit);
    }
    return i;
  }

template<class sigType>
  ostream& operator<<(ostream& o, const Sample<sigType>& s) {
    if  (! s.fPresent)
      return o << "#(" << s.fValue << ")";
    else
      return o << s.fValue;
  }

#endif
//  End of file Sample.C
