#ifndef _CLASS_SAMPLE_H_
#define _CLASS_SAMPLE_H_

//
//  File Sample.H
//
//  September 3 1992 - August 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "common.H"

class PureSample {
  private:
    SBoolean fPresent;        // Holds sample status

  public:
    PureSample();            // Initializes a new PureSample to not present
    PureSample(const SBoolean& b);
    PureSample(const PureSample& s);
    virtual bool isPresent() const;                    // Returns sample status
    virtual PureSample& operator=(const SBoolean& newVal); // Set the sample value.
    virtual PureSample& operator=(const PureSample& newSamp); // Copy a sample (this is for g++).
    virtual void reset();    // Resets sample status to not present
//
// Methods for sample input and output
//
    friend istream& operator>>(istream& i, PureSample& s);
    friend ostream& operator<<(ostream& o, const PureSample& s);
};

extern const PureSample Present;
extern const PureSample NotPresent;

template<class sigType>
  class Sample {        // Represents signal sample (status and value)
    private:
      SBoolean fPresent;    // Holds sample status
      sigType fValue;      // Sample value

    public:
      Sample();            // Initializes a new Sample to not present
      Sample(const sigType& val);
      Sample(const Sample<sigType>& s);
      virtual bool isPresent() const;    // Returns sample status
      virtual Sample<sigType>& operator=(const sigType& newVal); // Set the sample value.
      virtual Sample<sigType>& operator=(const Sample<sigType>& newSamp); // Copy Sample (this is for g++).
      virtual const sigType& value() const;  // Yields sample value
      virtual void reset();                  // Resets sample status to not present
//
// Methods for sample input and output
//
      friend istream& operator>>(istream& i, Sample<sigType>& s);
      friend ostream& operator<<(ostream& o, const Sample<sigType>& s);
  };

#ifdef __GNUG__
#include "Sample.C"
#endif

#endif

// End of file Sample.H
