#ifndef _CLASS_SIGNAL_C_
#define _CLASS_SIGNAL_C_

//
//  File Signal.C
//
//  September 3 1992 - November 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Signal.H"

// Signal definitions
//
template<class sigType>
  Signal<sigType>::Signal(const Synchronous* s, const char* name)
                  : VSignal(s, name) {
  }

template<class sigType>
  Sample<sigType>& Signal<sigType>::value() {
    return fValue;
  }

// InSignal definitions
//
template<class sigType>
  InSignal<sigType>::InSignal(const Synchronous* s, const char* name,
                              const bool dep)
                    : Signal<sigType>(s, name),
                      InputSignal(s, name, dep),
                      VSignal(s, name) {
  }

template<class sigType>
  void InSignal<sigType>::operator<<(OutSignal<sigType>& s) {
    connectTo(&s);
  }

template<class sigType>
  void InSignal<sigType>::update() {
    if ((source != 0) && (!source->isDead())) // If signal is connected
      fValue = ((OutSignal<sigType>*)source)->value();
                              // get next sample from source
    else {
      if (source != 0)        // If source is dead
        source = 0;           // Forget about it
    }
  }

// OutSignal definitions
//

template<class sigType>
  OutSignal<sigType>::OutSignal(const Synchronous *s, const char* name,
                                const bool dep)
                    : Signal<sigType>(s, name),
                      OutputSignal(s, name, dep),
                      VSignal(s, name)
  {
  }

#endif
// End of file Signal.C
