#ifndef _CLASS_SIGNAL_H_
#define _CLASS_SIGNAL_H_

//
//  File Signal.H
//
//  September 3 1992 - October 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "common.H"
#include "Sample.H"
#include "Connect.H"

class Synchronous;

class VSignal {                // Represents a signal in a module
  public:
    VSignal(const Synchronous*, const char* name);
    virtual const char* name() const;    // Yields signal name
    virtual void chown(const Synchronous*);  // Change owner
    virtual void rename(const char *newname);// Change name
    friend ostream& operator<<(ostream&,const VSignal&);
  protected:
    VSignal();
    const Synchronous *fOwner;               // Module using the signal
    const char* fName;
  private:
    VSignal(const VSignal&) ;
};

class OutputSignal : virtual public VSignal {
  public:
    OutputSignal(const Synchronous* s, const char* name,
                 const bool dep = true);
    virtual bool checkLoopFor(const Synchronous*) const;
    virtual unsigned long cref() const;
    virtual OutputSignal* inccref();
    virtual void deccref();
    virtual bool isDead() const;
  protected:
    const bool dep;
    unsigned long refcount;
  private:
    OutputSignal(const OutputSignal&) ;
};

class InputSignal : virtual public VSignal {
  public:
    InputSignal(const Synchronous* s, const char* name,
                const bool dep = true);
    virtual bool checkLoopThrough(const OutputSignal&) const;
                                // returns true if a loop exists
    virtual bool checkLoopFor(const Synchronous*) const;
                                // returns true if a loop exists
    virtual void unplug();      // Unplug the signal from its source
    virtual void checkSource(); // Check if source signal is still available
  protected:
    OutputSignal* source;		// != 0 if signal is connected to an output sig
    const bool dep;
    virtual void connectTo(OutputSignal*, const bool doIt = false);
    friend void Connect::doConnect() const;
  private:
    InputSignal(const InputSignal&) ;
};

class PureSignal : virtual public VSignal {  // Represents a pure signal
  public:
    PureSignal(const Synchronous*, const char* name);
    virtual PureSample& value();  // Yields current sample of signal
  protected:
    PureSample fValue;
  private:
    PureSignal(const PureSignal&) ;
};

template<class sigType>
  class Signal : virtual public VSignal {
                            // Represents a signal with a typed value
    public:
      Signal(const Synchronous* s, const char* name);
      virtual Sample<sigType>& value();  // Yields the current sample of signal
    protected:
      Sample<sigType> fValue;            // Current sample of signal
    private:
      Signal(const Signal<sigType> &) ;
  };

class PureOutSignal : public PureSignal, public OutputSignal {
  public:
    PureOutSignal(const Synchronous*, const char* name,
                  const bool dep = true);
  private:
    PureOutSignal(const PureOutSignal&) ;
};

template<class sigType>
  class OutSignal : public Signal<sigType>, public OutputSignal {
    public:
      OutSignal(const Synchronous* s, const char* name,
                const bool dep = true);
    private:
      OutSignal(const OutSignal<sigType> &) ;
  };

class PureInSignal : public PureSignal, public InputSignal {
  public:
    PureInSignal(const Synchronous*, const char* name,
                 const bool dep = true);
    virtual void operator<<(PureOutSignal&);
    virtual void update();
  private:
    PureInSignal(const PureInSignal&) ;
};

template<class sigType>
  class InSignal : public Signal<sigType>, public InputSignal {
    public:
      InSignal(const Synchronous*, const char* name,
               const bool dep = true);
      virtual void operator<<(OutSignal<sigType>&);
                                          // Connect to an output signal
      virtual void update();              // Update to next sample of signal
    private:
      InSignal(const InSignal<sigType> &) ;
  };

#ifdef __GNUG__
#include "Signal.C"
#endif

#endif

// End of file Signal.H
