//
//  File Synchronous.C
//
//  September 3 1992 - August 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Synchronous.H"
#include "SScheduler.H"
#include "Signal.H"

#include <iostream.h>

Synchronous::Synchronous(const char* name, const bool schedule)
            : fName(name), fClock(0), dead(false)
{
  if (schedule && (theScheduler != (SScheduler*)0)) {
    fClock = theScheduler;
    fClock->manage(this);
  } else
    fClock = 0;
}

const char* Synchronous::name() const {
  return fName;
}

Synchronous::~Synchronous() {
// Cannot call a virtual method (outSigList in safedelete) from a destructor!
//  if (!safedelete())
//    cerr << "# Warning: Object " << fName << " destroyed while its output signals are still in use." << endl;
//  die();
}

void Synchronous::endInstant() {  // Nothing special to do
}

void Synchronous::killSched() const {  // Kill the Scheduler to exit the application
  if (fClock != 0)
    fClock->stop();
}

SScheduler* Synchronous::clock() const {
  return fClock;
}

void Synchronous::react() {
  setInputs();        // Get input signal value
  activate();         // Compute new state and signal values
}

const InputSignal** Synchronous::waitSigList() const {
  return inSigList();  // Default behaviour: all signals are awaited
}

bool Synchronous::safedelete() const {
  const OutputSignal** outList = outSigList();
  int i = 0;
  while(outList[i] != 0) {
    if (outList[i++]->cref() != 0) {
      delete[] outList;
      return false;
    }
  }
  delete[] outList;
  return true;
}

void Synchronous::die() {
  dead = true;
}

bool Synchronous::isDead() const {
  return dead;
}

bool Synchronous::dependsOn(const Synchronous& s) const {
  const InputSignal **inList = inSigList();
  int i = 0;
  while(inList[i] != 0) {
    if (inList[i++]->checkLoopFor(&s)) {
      delete[] inList;
      return true;
    }
  }
  delete[] inList;
  return false;
}

Synchronous& Synchronous::operator<<(Synchronous& s) {
  if (! preferedBefore(s))
    areSup[areSup.size()] = &s;
  return s;
}

bool Synchronous::preferedBefore(const Synchronous& s) const {
  for(int i = 0; i < areSup.size(); i++) {
    if ((areSup.get(i) == &s) || areSup.get(i)->preferedBefore(s))
      return true;
  }
  return false;
}

bool Synchronous::operator>(const Synchronous& s) const {
  if (dependsOn(s))
    return true;
  if (s.dependsOn(*this))
    return false;
  if (preferedBefore(s))
    return false;
  else
    return true;
}

bool Synchronous::operator<(const Synchronous& s) const {
  return s > (*this);
}

bool Synchronous::operator>=(const Synchronous& s) const {
  return !((*this) < s);
}

bool Synchronous::operator<=(const Synchronous& s) const {
  return !((*this) > s);
}

// End of file Synchronous.C
