#ifndef _CLASS_SYNCHRONOUS_H_
#define _CLASS_SYNCHRONOUS_H_

//
//  File Synchronous.H
//
//  September 3 1992 - November 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "common.H"

class SScheduler;
class InputSignal;
class OutputSignal;

#include "Array.H"

//
//  Synchronous represent synchronous modules
//

class Synchronous {
  private:
    const char* fName;
    Array<Synchronous> areSup;   // Array of Synchronous that the user
                                 // would like to make react after me.
    Synchronous(const Synchronous&);

  protected:
    SScheduler *fClock;           // Pointer to the clock
    bool dead;

  public:
    Synchronous(const char* name, const bool schedule = true);
    virtual ~Synchronous();
    virtual void killSched() const;    // Kill the Scheduler
    virtual SScheduler* clock() const;  // Get the clock
    virtual void endInstant();       // The instant has ended
    virtual void react();            // Compute outputs versus inputs
    virtual void resetOutputs() = 0; // Reset output signals to not emitted.
    virtual void resetInputs() = 0;  // Reset input signals to not present.
    virtual void setInputs() = 0;    // Prepare to react!
    virtual void activate() = 0;     // Do the real job.
    virtual void backup() = 0;       // Save your state
    virtual void restore() = 0;      // Restore saved state if any
    virtual const InputSignal** inSigList() const = 0;
                                 // Return the list of input signals in the module
    virtual const OutputSignal** outSigList() const = 0;
                                 // Return the list of output signals in the module
    virtual const InputSignal** waitSigList() const;
                                 // Return the list of awaited input signals
    virtual bool safedelete() const;// Is it safe to delete this module?
    virtual const char* name() const;  // Yields module name
    virtual void die();                // Enter pre-destruction state
    virtual bool isDead() const;    // Tell if object has still a future
    virtual bool dependsOn(const Synchronous& s) const;
    virtual Synchronous& operator<<(Synchronous& s);
    virtual bool preferedBefore(const Synchronous& s) const;
    virtual bool operator>(const Synchronous& s) const;
                                 // Tells if (*this) must react after s
    virtual bool operator<(const Synchronous& s) const;
                                 // Tells if (*this) must react before s
    virtual bool operator>=(const Synchronous& s) const;
                                 // Tells if (*this) must react after s or the
                                 // order of reaction of (*this) and s does not matter.
    virtual bool operator<=(const Synchronous& s) const;
                                 // Tells if (*this) must react before s or the
                                 // order of reaction of (*this) and s does not matter.
};

#endif

// End of file Synchronous.H
