#ifndef _CLASS_TIMEMODULE_H_
#define _CLASS_TIMEMODULE_H_

//
//  File TimeModule.H
//
//  September 3 1992 - September 1996
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1996.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "common.H"

#include <time.h>

#ifndef __osf__
#include <sys/timeb.h>
extern "C" ftime(struct timeb *);
#endif

#include "Synchronous.H"
#include "Signal.H"


//
// The TimeModule module gives the second, minute and hour signals
//
class TimeModule : virtual public Synchronous {
  public:
    TimeModule(const char* name = "TimeModule",
               const bool schedule = true);
    ~TimeModule();
    const InputSignal** inSigList() const;
    const OutputSignal** outSigList() const;
    virtual OutSignal<int>& milliSecond();
    virtual OutSignal<int>& centiSecond();
    virtual OutSignal<int>& deciSecond();
    virtual OutSignal<int>& second();
    virtual OutSignal<int>& minute();
    virtual OutSignal<int>& hour();
    void resetOutputs();
    void resetInputs();
    void setInputs();
    void activate();
    virtual void backup();
    virtual void restore();
//    TimeModule& operator=(const TimeModule&);
  private:
    class TimeData {
    	public:
        Sample<int> fMilliSecond;
        Sample<int> fCentiSecond;
        Sample<int> fDeciSecond;
        Sample<int> fSecond;
        Sample<int> fMinute;
        Sample<int> fHour;
    };
    OutSignal<int> fMilliSecond;
    OutSignal<int> fCentiSecond;
    OutSignal<int> fDeciSecond;
    OutSignal<int> fSecond;
    OutSignal<int> fMinute;
    OutSignal<int> fHour;
    bool fBackuping;
    bool fRestoring;
    TimeData *pf_backup_copy_;
    TimeModule(const TimeModule&);
#ifdef __osf__
    timespec fClock;
#else
    timeb fClock;
#endif
};

#endif

