//
//  File Point.C
//
//  September 3 1992 - August 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync demos
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Point.H"
#include <math.h>

#if defined(__DECCXX) || defined(__GNUG__)
# include <stdlib.h>
#endif

Point::Point()
   : px(0), py(0)
{}

Point::Point(const int xi, const int yi)
   : px(xi), py(yi)
{}

int Point::operator==(const Point& a) const {
  return (px == a.px) && (py == a.py);
}

Point Point::operator-(const Point& a) const {
  Point retVal(px - a.px , py - a.py);
  return retVal;
}

Point Point::operator+(const Point& a) const {
  Point retVal(px + a.px , py + a.py);
  return retVal;
}

Point Point::nextToward(const Point& a) const {
  int dx=0, dy=0;
  Point depl;
  double t;
  
  if (a.py == py) {
    if (a.px > px)
      dx = 1;
    else if (a.px < px)
      dx = -1;
  } else {
    t = ((double)(a.px - px)/(double)abs(a.py - py));
    if (t >= 0.5)
      dx = 1;
    else if (t <= -0.5)
      dx = -1;
  }
  if (a.px == px) {
    if (a.py > py)
      dy = 1;
    else if (a.py < py)
      dy = -1;
  }else {
    t = ((double)(a.py - py)/(double)abs(a.px - px));
    if (t >= 0.5)
      dy = 1;
    else if (t <= -0.5)
      dy = -1;
  }
  depl = Point(dx,dy);
  return depl + *this;
}

int Point::x() const {
  return px;
}

int Point::y() const {
  return py;
}

double Point::dist(const Point& a) const {
  return hypot(double(px - a.px), double(py - a.py));
}

ostream& operator<<(ostream& o, const Point& a) {
  return(o << "(" << a.px << "," << a.py << ")");
}

istream& operator>>(istream& i, Point& a) {
  char tmp(' ');
  Point p;

  i >> tmp;
  if (tmp != '(') {
    i.putback(tmp);
    i.clear(ios::failbit | i.rdstate());
    return i;
  }
  i >> p.px;
  if (!i.good()) {
    i.clear(ios::badbit | i.rdstate());
    return i;
  }
  i >> tmp;
  if (tmp != ',') {
    i.clear(ios::badbit | i.rdstate());
    return i;
  }
  i >> p.py;
  if (!i.good()) {
    i.clear(ios::badbit | i.rdstate());
    return i;
  }
  i >> tmp;
  if (tmp != ')') {
    i.clear(ios::badbit | i.rdstate());
    return i;
  }
  a = p;
  return i;
}
