//
//  File XT.C
//
//  September 3 1992 - August 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync demos
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "turtle.H"
#include "XTurtle.H"
#include "XGC.H"

class STurtle {
  public:
    turtle turt;
    Delay<Point> del;
    XTurtle xt;
    STurtle(turtleConsts& pos, XWindow& w, XGC& gc, int step, int margin,
            const char* name = "Sturtle")
      : turt(pos, name),
        del(&(pos.initPos),Composite::subname(name, "delay")),
        xt(Composite::subname(name, "Xturtle"),w,gc,step,margin,4)
    {
      xt.probe() << turt.pos();
      turt.cible() << del.output();
    }
    
    InSignal<Point>& in() {
      return del.input();
    }
    
    OutSignal<Point>& pos() {
      return turt.pos();
    }
};


int main(int argc, char**) {
  turtleConsts initConsts;
  
  XDisplay d;
  XScreen s(d);
  XWindow root(s);
  
  XSizeHints h;
  boolean done = false;
  boolean debug = false;
  const int margin = 10;
  const int wsize = 200;
  const int tpos = 20;
  const int step = wsize/tpos;
  
  h.x = 30; h.y = 30; h.width = wsize+2*margin; h.height = wsize+2*margin;
  h.min_width = wsize+2*margin; h.min_height = wsize+2*margin;
  h.max_height = wsize+2*margin; h.max_width = wsize+2*margin;
  h.flags = PPosition | PSize | PMinSize | PMaxSize;

  
  XWindow w(root, h, "Four turtle on a clock");
  w.mapRaised();
  
  XGC gc(w);
  gc.setForeground(s.blackPixel());
  gc.setBackground(s.whitePixel());
  
  cerr << "This program shows 4 turtle in a window. Each turle follows" << endl;
  cerr << "the next one, the last turtle following the first one. To avoid" << endl;
  cerr << "a causality loop, a delay must be inserted. To preserve the symetry" << endl;
  cerr << "of the system, a delay has been inserted between each pair of " << endl;
  cerr << "followed/following turtle." << endl;
  cerr << "Call this program with one argument to run the clock in simulation mode." << endl << endl;
  
  if (argc > 1)
    debug = true;
  SScheduler theClock("theSched",debug);

  initConsts.initPos = Point(0,0);
  STurtle t1(initConsts,w, gc, step, margin,"t1");

  initConsts.initPos = Point(0,tpos);
  STurtle t2(initConsts,w, gc, step, margin,"t2");

  initConsts.initPos = Point(tpos,tpos);
  STurtle t3(initConsts,w, gc, step, margin,"t3");

  initConsts.initPos = Point(tpos,0);
  STurtle t4(initConsts,w, gc, step, margin,"t4");

  
  t1.in() << t2.pos();
  t2.in() << t3.pos();
  t3.in() << t4.pos();
  t4.in() << t1.pos();

  XFlush(d.xdisplay());
  while(theClock.tick()) {
    XFlush(d.xdisplay());
    if (debug) {
      cerr << "t1: pos=";
      operator<<(cerr, t1.turt.pos().value());
      cerr << ", cible=" << t1.turt.cible().value() << endl;
      cerr << "t2: pos=" << t2.turt.pos().value() << ", cible=" << t2.turt.cible().value() << endl;
      cerr << "t3: pos=" << t3.turt.pos().value() << ", cible=" << t3.turt.cible().value() << endl;
      cerr << "t4: pos=" << t4.turt.pos().value() << ", cible=" << t4.turt.cible().value() << endl;
    }
  }
  return 0;
}
