//
//  File XTurtle.C
//
//  September 3 1992 - August 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync demos
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "XTurtle.H"

XTurtle::XTurtle(const char* name, const XWindow& w,
        const XGC& gc, int step, int margin,
        int radius, const bool sched)
    : Synchronous(name, sched),
      fProbe(this,"Probe"),
      myWind(w),
      myGC(gc),
      myStep(step),
      myMargin(margin),
      myRadius(radius),
      hasLast(false),
      _pf_backup_copy_(0)
{}

const InputSignal** XTurtle::inSigList() const {
  const InputSignal** inList = (const InputSignal**) new (InputSignal*[2]);
  inList[0] = &fProbe;
  inList[1] = 0;
  return inList;
}

const OutputSignal** XTurtle::outSigList() const {
  const OutputSignal** outList = (const OutputSignal**) new (OutputSignal*[1]);
  outList[0] = 0;
  return outList;
}

InSignal<Point>& XTurtle::probe() {
  return fProbe;
}

void XTurtle::resetOutputs() {
}

void XTurtle::resetInputs() {
  fProbe.value().reset();
}

void XTurtle::setInputs() {
  fProbe.update();
}

void XTurtle::activate() {
  if (fProbe.value().isPresent()) {
    if (hasLast) {
      XDrawLine(myWind.screen().display().xdisplay(),
            myWind.xwindow(),
            myGC.xgc(),
            myMargin+myStep*lastPos.x(), myMargin+myStep*lastPos.y(),
            myMargin+myStep*fProbe.value().value().x(),
            myMargin+myStep*fProbe.value().value().y());
      lastPos = fProbe.value().value();
    } else {
      XDrawPoint(myWind.screen().display().xdisplay(),
             myWind.xwindow(),
             myGC.xgc(),
             myMargin+myStep*fProbe.value().value().x(),
             myMargin+myStep*fProbe.value().value().y());
      lastPos = fProbe.value().value();
      hasLast = true;
    }
    if (myRadius > 1) {
      XFillArc(myWind.screen().display().xdisplay(),
               myWind.xwindow(),
               myGC.xgc(),
               myMargin+myStep*fProbe.value().value().x() - myRadius/2,
               myMargin+myStep*fProbe.value().value().y() - myRadius/2,
               myRadius, myRadius, 0, 360*64);
    }
  }
}

void XTurtle::backup() {
	if (_pf_backup_copy_ == 0)
		_pf_backup_copy_ = new XTurtle(name(),myWind,myGC,myStep,myMargin,myRadius,false);
	_pf_backup_copy_->lastPos = lastPos;
	_pf_backup_copy_->hasLast = hasLast;
	_pf_backup_copy_->fProbe.value() = fProbe.value();
}

void XTurtle::restore() {
	if (_pf_backup_copy_ != 0) {
		lastPos = _pf_backup_copy_->lastPos;
		hasLast = _pf_backup_copy_->hasLast;
		fProbe.value() = _pf_backup_copy_->fProbe.value();
	}
}
