//
//  File AsyncEmitH.C
//
//  September 3 1992 - January 21 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "AsyncEmitH.H"
#include "AsyncReceptH.H"

//template<class sigType>
//  const int AsyncEmitH<sigType>::skChunkSize = 10;

template<class sigType>
  AsyncEmitH<sigType>::AsyncEmitH(const char* name, const boolean schedule,
                                  const int chunkSize)
                     : Synchronous(name, schedule),
                       fIn(this, "out"),
                       nbDest(0),
                       kChunkSize(chunkSize)
{
 fDestList = new (AsyncReceptH<sigType>*[kChunkSize]);
 size = kChunkSize;
 fDestList[0] = 0;
}

template<class sigType>
  AsyncEmitH<sigType>::~AsyncEmitH(){
    delete[] fDestList;
    nbDest = 0;
  }

template<class sigType>
  int AsyncEmitH<sigType>::index(const AsyncReceptH<sigType>& a) {
    for(int i = 0; i < nbDest; i++) {
      if (fDestList[i] == &a)
        return i;
    }
    return -1;
  }

template<class sigType>
  void AsyncEmitH<sigType>::subscribe(AsyncReceptH<sigType>& a) {
    if (index(a) < 0) {
      if (size <= (nbDest + 1)) {
        AsyncReceptH<sigType>** tmp = fDestList;
        fDestList = new (AsyncReceptH<sigType>* [size+kChunkSize]);
        size += kChunkSize;
        for (int i = 0; i < nbDest; i++)
          fDestList[i] = tmp[i];
        delete[] tmp;
      }
      fDestList[nbDest++] = &a;
      fDestList[nbDest] = 0;
    }
  }

template<class sigType>
  void AsyncEmitH<sigType>::unsubscribe(AsyncReceptH<sigType>& a) {
    int idx = index(a);
    if (idx >=0) {
      for (int i = idx; i < nbDest; i++)
        fDestList[i] = fDestList[i+1];
      nbDest --;

      if ((size - nbDest) > kChunkSize) {
        AsyncReceptH<sigType>** tmp = fDestList;
        fDestList = new (AsyncReceptH<sigType>* [size-kChunkSize]);
        size -= kChunkSize;
        for (int i = 0; i < nbDest; i++)
          fDestList[i] = tmp[i];
        delete[] tmp;
      }
      fDestList[nbDest] = 0;
    }
  }

template<class sigType>
  const InputSignal** AsyncEmitH<sigType>::inSigList() const {
    const InputSignal** inList;
  
    inList = new (const InputSignal*[2]);
    inList[0] = &fIn;
    inList[1] = 0;
    return inList;
  }

template<class sigType>
  const OutputSignal** AsyncEmitH<sigType>::inSigList() const {
    const OutputSignal** outList;
  
    outList = new (const OutputSignal*[1]);
    outList[0] = 0;
    return outList;
  }

template<class sigType>
  InSignal<sigType>& AsyncEmitH<sigType>::input() {
    return fIn;
  }

template<class sigType>
  InSignal<sigType>& AsyncEmitH<sigType>::operator()() {
    return fIn;
  }

template<class sigType>
  void AsyncEmitH<sigType>::resetOutputs() {
  }

template<class sigType>
  void AsyncEmitH<sigType>::setInputs() {
    fIn.update();
  }

template<class sigType>
  void AsyncEmitH<sigType>::activate() {
    if (fIn.value().isPresent()) {
      for(int i = 0; i < nbDest; i++)
        fDestList[i]->put(fIn.value().value());
    }
  }
