//
//  File AsyncReceptH.C
//
//  September 3 1992 - January 21 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "AsyncReceptH.H"
#include "AsyncEmitH.H"

//template<class sigType>
//  const int AsyncReceptH<sigType>::skChunkSize = 10;

template<class sigType>
  AsyncReceptH<sigType>::AsyncReceptH(const char* name, const int chunkSize,
                                      const boolean schedule)
                        : Synchronous(name, schedule),
                          fOut(this, "out"),
                          fSource(0),
                          first(0), last(0), size(0), kChunkSize(chunkSize)
{
  fHist = new sigType[kChunkSize];
  size = kChunkSize;
  access.signal();
}

template<class sigType>
  AsyncReceptH<sigType>::~AsyncReceptH() {
    access.wait();
    if (fSource != 0)
      fSource->unsubscribe(*this);
    delete[] fHist;
    size = 0;
    first = 0;
    last = 0;
    access.signal();
  }

template<class sigType>
  void AsyncReceptH<sigType>::getFrom(AsyncEmitH<sigType> *source) {
    if (fSource != 0)
      fSource->unsubscribe(*this);
    fSource = source;
    if (fSource != 0)
      fSource->subscribe(*this);
  }

template<class sigType>
  void AsyncReceptH<sigType>::put(const sigType& val) {
    access.wait();
    if ((last+1)%size == first) {
      sigType* tmp = fHist;
      fHist = new sigType[size+kChunkSize];
      int i = 0;
      int j;
      while((j = (first+i)%size) != last)
        fHist[++i] = tmp[j+1];
      first = 0;
      last = i;
      size += kChunkSize;
    }
    last = (last+1) % size;
    fHist[last] = val;
    access.signal();
  }

template<class sigType>
  const InputSignal** AsyncReceptH<sigType>::inSigList() const {
    const InputSignal** inList;
  
    inList = new (const InputSignal*[1]);
    inList[0] = 0;
    return inList;
  }

template<class sigType>
  const OutputSignal** AsyncReceptH<sigType>::outSigList() const {
    const OutputSignal** outList;
  
    outList = new (const OutputSignal*[2]);
    outList[0] = &fOut;
    outList[1] = 0;
    return outList;
  }

template<class sigType>
  OutSignal<sigType>& AsyncReceptH<sigType>::output() {
    return fOut;
  }

template<class sigType>
  OutSignal<sigType>& AsyncReceptH<sigType>::operator()() {
    return fOut;
  }

template<class sigType>
  void AsyncReceptH<sigType>::resetOutputs() {
    fOut.value().reset();
  }

template<class sigType>
  void AsyncReceptH<sigType>::setInputs() {
  }

template<class sigType>
  void AsyncReceptH<sigType>::activate() {
    access.wait();
    if (first != last) {
      first = (first + 1)%size;
      fOut.value() = fHist[first];
    }
    access.signal();
  }
