//
//  File HSort.C
//
//  September 3 1992 - February 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "HSort.H"
#include "Array.H"

// Heap sort
//

template<class T>
  HSort<T>::HSort(Array<T>& a)
         : a(a)
{
}

template<class T>
  HSort<T>::~HSort() {
}

template<class T>
  Array<T>& HSort<T>::array() {
  return a;
}

template<class T>
  void HSort<T>::sort() {
  int sz = a.size() - 1;
  int maxelt;
  T* max;
  
  for(maxelt = father(sz); maxelt >= root(); maxelt--)
    buildheap(maxelt);
  for(maxelt = sz; maxelt > root(); a[maxelt+1] = max)
    removemax(maxelt, max);
  
}

template<class T>
  void HSort<T>::removemax(int& maxelt, T*& max) {
  max = a[root()];        // Maximum is root of tree
  a[root()] = a[maxelt];  // Put last leaf at root
  maxelt--;               // There is one less element
  int i = root();         // Re-built the new heap
  int j;
  while (i <= father(maxelt)) { // While root is not a leaf
    if ((maxelt == lson(i)) || (greater(a[lson(i)], a[rson(i)])))
      j = lson(i);         // Get greater son of i
    else
      j = rson(i);
    if (greater(a[j], a[i])) { // If greater son is more than root
      a.swap(i, j);          // swap them
      i = j;
    } else
      break;
  }
}

template<class T>
  void HSort<T>::buildheap(int i) {
  int sz = a.size() - 1;
  int j;
  boolean done = false;
  
  do {
    if ((rson(i) > sz) || (greateq(a[lson(i)], a[rson(i)])))
      j = lson(i);             // greater son of i
    else
      j = rson(i);
    if (greater(a[j], a[i])) { // If smaller son > father
      a.swap(i,j);             // exchange
      i = j;                   // Go on with new sub-tree
      done = boolean(i > father(sz)); // done if i is a leaf
    } else
      done = true;
  } while (!done);
}

template<class T>
  boolean HSort<T>::greater(const T* const t1, const T* const t2) const {
  if ((t1 == t2) || (t2 == 0))
    return false;
  if (t1 == 0)
    return true;
  return boolean((*t1) > (*t2));
}

template<class T>
  boolean HSort<T>::greateq(const T* const t1, const T* const t2) const {
  if ((t1 == t2) || (t1 == 0))
    return true;
  if (t2 == 0)
    return false;
  return boolean((*t1) >= (*t2));
}

