//
//  File List.C
//
//  September 3 1992 - February 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "List.H"

template<class T>
Link<T>::Link(const T* elem)
     : fValue(elem),
       fNext(0)
{
}

template<class T>
Link<T>::~Link() {
  if (fNext != 0)
    delete fNext;
}

template<class T>
T* Link<T>::value() const {
  return fValue;
}

template<class T>
Link<T>* Link<T>::next() const {
  return fNext;
}

template<class T>
void Link<T>::insert(Link<T>* next) {
  if (next != 0)
    next->fNext = fNext;
  fNext = next;
}

template<class T>
void Link<T>::append(Link<T>* next) {
  if (fNext == 0)
    fNext = next;
  else {
    Link<T> *tmp = fNext;
    while(tmp->fNext != 0)
      tmp = tmp->fNext;
    tmp->fNext = next;
  }
}

template<class T>
void Link<T>::rmnext() {
  if (fNext != 0) {
    Link<T> *tmp = fNext;
    fNext = fNext->fNext;
    tmp->insert(0);
    delete tmp;
  }
}

template<class T>
int Link<T>::operator==(const Link<T>& l) {
  return (value == l.value);
}

template<class T>
int Link<T>::operator<(const Link<T>& l) {
  return (value < l.value);
}

template<class T>
int Link<T>::operator>(const Link<T>& l) {
  return (value > l.value);
}



template<class T>
List<T>::List()
     : fLen(0),
       list(0)
{
}

template<class T>
List<T>::~List() {
  if (list != 0)
    delete list;
  fLen = 0;
}

template<class T>
void List<T>::append(const T* elem) {
  if (list == 0) {
    list = new Link<T>(elem);
    fLen = 1;
  } else {
    list->append(new Link<T>(elem));
    fLen ++;
  }
}

template<class T>
void List<T>::append(const List<T> *l) {
  if (list == 0)
    list = l->list;
  else
    list->append(l->list);
  fLen += l->length();
}

template<class T>
void List<T>::remove(const T* elem) {
  Link<T> *tmp, *cur;
  while ((list != 0) && (list->value == elem)) {
    tmp = list;
    list = list->next();
    tmp->insert(0);
    delete tmp;
    fLen --;
  }
  tmp = list;
  while(tmp != 0) {
    while ((tmp->next != 0) && (tmp->next()->value() == elem)) {
      tmp->rmnext();
      fLen --;
    }
    tmp = tmp->next();
  }
}

template<class T>
void List<T>::emptyList() {
  delete list;
  fLen = 0;
}

template<class T>
int List<T>::length() {
  return fLen;
}

template<class T>
int List<T>::index(const T* elem) {
  Link<T> *tmp = list;
  int i = 0;
  while(tmp != 0) {
    if (tmp->value() == elem)
      return i;
    else {
      tmp = tmp->next();
      i++;
    }
  }
  return -1;
}

template<class T>
Link<T>* List<T>::get(const int i) {
  if ((i < 0) || (i >= fLen))
    return 0;
  else {
    Link<T> *tmp = list;
    for(int j = 0; j < i; j++)
      tmp = tmp->next();
    return tmp;
  }
}

template<class T>
void List<T>::remove(const int i) {
  Link<T>* tmp = get(i);
  if (tmp == 0)
    return;
  if (tmp == list) {
    list = list->next();
    tmp->insert(0);
    delete tmp;
    fLen --;
  } else {
    tmp = get(i - 1);
    tmp->rmnext();
    fLen --;
  }
}



template<class T>
ListIter<T>::ListIter(List<T>& l, const int i) 
            : fList(&l)
{
  if ((i < 0) || (i >= fList->length()))
    fOffset = -1;
  else
    fOffset = i;
}

template<class T>
ListIter<T>::ListIter(List<T>& l, const T* elem) 
            : fList(&l)
{
  fOffset = fList->index(T);
}

template<class T>
ListIter<T>::~ListIter() {
  fList = 0;
  fOffset = -1;
}

template<class T>
T* ListIter<T>::current() {
  Link<T>* tmp = fList->get(fOffset);
  if (tmp == 0)
    return 0;
  else
    return tmp->value();
}

template<class T>
T* ListIter<T>::next() {
  Link<T>* tmp = fList->get(fOffset + 1);
  if (tmp == 0)
    return 0;
  else {
    fOffset ++;
    return tmp->value();
  }
}

template<class T>
T* ListIter<T>::prev() {
  Link<T>* tmp = fList->get(fOffset - 1);
  if (tmp == 0)
    return 0;
  else {
    fOffset --;
    return tmp->value();
  }
}

template<class T>
void ListIter<T>::toStart() {
  fOffset = 0;
}

template<class T>
void ListIter<T>::toEnd() {
  fOffset = fList->length() - 1;
}

template<class T>
boolean ListIter<T>::beforeStart() {
  return boolean(fOffset == -1);
}

template<class T>
boolean ListIter<T>::afterEnd() {
  return boolean(fOffset == fList->length());
}

template<class T>
void ListIter<T>::remove() {
  fList->remove(fOffset);
}

template<class T>
void ListIter<T>::seek(const int i) {
  fOffset = i;
  if (fOffset > fList->length())
    fOffset = fList->length();
  if (fOffset < 0)
    fOffset = -1;
}

template<class T>
int ListIter<T>::offset() {
  return fOffset;
}

template<class T>
List<T>& ListIter<T>::getList() {
  return *fList;
}

