//
//  File List.H
//
//  September 3 1992 - February 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#ifndef _LIST_H_
#define _LIST_H_

template<class T> class Link {
  public:
    Link(const T* elem);
    virtual ~Link();
    virtual T* value() const;
    virtual Link<T> *next() const;
    virtual void insert(Link<T> *next);
    virtual void append(Link<T> *next);
    virtual void rmnext();
    virtual int operator==(const Link<T>& l);
    virtual int operator<(const Link<T>& l);
    virtual int operator>(const Link<T>& l);
  protected:
    const T* fValue;
    Link<T> *fNext;
};

template<class T> class List {
  public:
    List();
    virtual ~List();
    virtual void append(const T* elem);
    virtual void append(const List<T> *l);
    virtual void remove(const T* elem);
    virtual void emptyList();
    virtual int length();
  protected:
    int fLen;
    Link<T> *list;
    virtual Link<T> *get(const int i);
    virtual void remove(const int i);
    virtual int index(const T* elem);
    friend ListIter<T>;
};

template<class T> class ListIter {
  public:
    ListIter(List<T>& l, const int i = 0);
    ListIter(List<T>& l, const T* elem);
    virtual ~ListIter();
    virtual T* current();
    virtual T* next();
    virtual T* prev();
    virtual void toStart();
    virtual void toEnd();
    virtual boolean beforeStart();
    virtual boolean afterEnd();
    virtual void remove();
    virtual void seek(const int i);
    virtual int offset();
    virtual List<T>& getList();
  private:
    List<T> *fList;
    int fOffset;
};

#endif
