//
//  File Demultiplexer.C
//
//  September 3 1992 - January 21 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Demultiplexer.H"

#include <iostream.h>
#include <strstream.h>

template<class sigType>
  Demultiplexer<sigType>::Demultiplexer(const char* name, const int nbOutputs,
                                        const boolean schedule)
                        : Synchronous(name, schedule),
                          fNbOutputs(nbOutputs),
                          fInput(this, "input"),
                          fSelect(this, "select")
  {
    ostrstream *buf;
    fOutputs = new (OutSignal<sigType>*[fNbOutputs + 1]);
    for(int i=0; i < fNbOutputs; i++) {
      buf = new ostrstream;
      (*buf) << "outSig" << i;
      fOutputs[i] = new OutSignal<sigType>(this, buf->str());
      delete buf;
    }
    fOutputs[fNbOutputs] = 0;
  }

template<class sigType>
  Demultiplexer<sigType>::~Demultiplexer() {
    for(int i = 0; i < fNbOutputs; i++)
      delete fOutputs[i];
    delete[] fOutputs;
  }

template<class sigType>
  void Demultiplexer<sigType>::resetOutputs() {
    for(int i=0; i < fNbOutputs; i++)
    fOutputs[i]->value().reset();
  }

template<class sigType>
  void Demultiplexer<sigType>::setInputs() {
    fInput.update();
    fSelect.update();
  }

template<class sigType>
  void Demultiplexer<sigType>::activate() {
    int path = fSelect.value().value();
    if ((path >= 0) && (path < fNbOutputs)) {
      if (fInput.value().isPresent())
        fOutputs[path]->value() = fInput.value().value();
    }
  }

template<class sigType>
  int Demultiplexer<sigType>::nbOutputs() {
    return fNbOutputs;
  }

template<class sigType>
  const InputSignal** Demultiplexer<sigType>::inSigList() const {
    const InputSignal ** inList;
  
    inList = new (const InputSignal*[3]);
    inList[0] = &fInput;
    inList[1] = &fSelect;
    inList[2] = 0;
    return inList;
  }

template<class sigType>
  const OutputSignal** Demultiplexer<sigType>::outSigList() const {
    const OutputSignal ** outList;
  
    outList = new (const OutputSignal*[fNbOutputs + 1]);
    for(int i=0; i <= fNbOutputs; i++)
      outList[i] = fOutputs[i];
    return outList;
  }

template<class sigType>
  InSignal<sigType>& Demultiplexer<sigType>::input() {
    return fInput;
  }

template<class sigType>
  InSignal<int>& Demultiplexer<sigType>::select() {
    return fSelect;
  }

template<class sigType>
  OutSignal<sigType>& Demultiplexer<sigType>::output(const int i) {
    if ((i >= 0) && (i < fNbOutputs))
      return *(fOutputs[i]);
    else {
      cerr << "# Index of output signal is out of range." << endl;
      exit(1);
    }
  }
