//
//  File Merger.C
//
//  September 3 1992 - January 21 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Merger.H"
#include "Synchronous.H"
#include "Signal.H"
#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>

template<class sigType>
  Merger<sigType>::Merger(const char* name, const int nbEntries,
                          const boolean schedule)
                  : Synchronous(name, schedule),
                    fOutput(this, "output"),
                    nbEntries(nbEntries)
{
  ostrstream *buf;
  fInputs = new (InSignal<sigType>*[nbEntries + 1]);
  for(int i = 0; i < nbEntries; i++) {
    buf = new ostrstream;
    (*buf) << "iSig" << i;
    fInputs[i] = new InSignal<sigType>(this,buf->str());
    delete buf;
  }
  fInputs[nbEntries] = 0;
}

template<class sigType>
  Merger<sigType>::~Merger() {
    for(int i = 0; i < nbEntries; i++)
      delete fInputs[i];
    delete[] fInputs;
}

template<class sigType>
  void Merger<sigType>::resetOutputs() {
    fOutput.value().reset();
  }

template<class sigType>
  void Merger<sigType>::setInputs() {
    for(int i=0; i < nbEntries; i++)
    fInputs[i]->update();
  }

template<class sigType>
  void Merger<sigType>::activate() {
    sigType tmpVal;
    boolean affected = false;
  
    for(int i=0; i < nbEntries; i++) {
      if (fInputs[i]->value().isPresent()) {
        if (affected)
          tmpVal = combine(tmpVal,fInputs[i]->value().value());
        else {
          tmpVal = fInputs[i]->value().value();
          affected = true;
        }
      }
    }
    if (affected)
      fOutput.value() = tmpVal;
  }

template<class sigType>
  const InputSignal** Merger<sigType>::inSigList() const {
    const InputSignal** inList;
  
    inList = new (const InputSignal*[nbEntries + 1]);
    for(int i=0; i < (nbEntries + 1); i++)
      inList[i] = fInputs[i];
    return inList;
  }

template<class sigType>
  const OutputSignal** Merger<sigType>::outSigList() const {
    const OutputSignal** outList;
  
    outList = new (const OutputSignal*[2]);
    outList[0] = &fOutput;
    outList[1] = 0;
    return outList;
  }

template<class sigType>
  int Merger<sigType>::nbInputs() {
    return nbEntries;
  }

template<class sigType>
  InSignal<sigType>& Merger<sigType>::input(const int i) {
    if (i < nbEntries)
      return *(fInputs[i]);
    else {
      cerr << "# Index of input signal out of range" << endl;
      exit(1);
    }
  }

template<class sigType>
  OutSignal<sigType>& Merger<sigType>::output() {
    return fOutput;
  }
