//
//  File PureDemult.C
//
//  September 3 1992 - January 21 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "PureDemult.H"

#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>

PureDemult::PureDemult(const char* name, const int nbOutputs,
                      const boolean schedule)
            : Synchronous(name, schedule),
              fNbOutputs(nbOutputs),
              fInput(this, "input"),
              fSelect(this, "select")
  {
    ostrstream *buf;
    fOutputs = new (PureOutSignal*[fNbOutputs + 1]);
    for(int i=0; i < fNbOutputs; i++) {
      buf = new ostrstream;
      (*buf) << "outSig" << i;
      fOutputs[i] = new PureOutSignal(this, buf->str());
      delete buf;
    }
    fOutputs[fNbOutputs] = 0;
  }

PureDemult::~PureDemult() {
  for(int i = 0; i < fNbOutputs; i++)
    delete fOutputs[i];
  delete[] fOutputs;
}

void PureDemult::resetOutputs() {
  for(int i=0; i < fNbOutputs; i++)
    fOutputs[i]->value().reset();
}

void PureDemult::setInputs() {
  fInput.update();
  fSelect.update();
}

void PureDemult::activate() {
  int path = fSelect.value().value();
  if ((path >= 0) && (path < fNbOutputs)) {
    if (fInput.value().isPresent())
      fOutputs[path]->value() = true;
  }
}

int PureDemult::nbOutputs() {
  return fNbOutputs;
}

const InputSignal** PureDemult::inSigList() const {
  const InputSignal ** inList;
  
  inList = new (const InputSignal*[3]);
  inList[0] = &fInput;
  inList[1] = &fSelect;
  inList[2] = 0;
  return inList;
}

const OutputSignal** PureDemult::outSigList() const {
  const OutputSignal ** outList;
  
  outList = new (const OutputSignal*[fNbOutputs + 1]);
  for(int i = 0; i <= fNbOutputs; i++)
    outList[i] = fOutputs[i];
  return outList;
}

PureInSignal& PureDemult::input() {
  return fInput;
}

InSignal<int>& PureDemult::select() {
  return fSelect;
}

PureOutSignal& PureDemult::output(const int i) {
  if ((i >= 0) && (i < fNbOutputs))
    return *(fOutputs[i]);
  else {
    cerr << "# Index of output signal is out of range." << endl;
    exit(1);
  }
}
