//
//  File PureMerge.C
//
//  September 3 1992 - January 21 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  which is copyright Ecole Superieure d'Electricite, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by the Ecole Superieure
//       d'Electricite and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//  Neither the name of the Ecole Superieure d'Electricite nor the names of its
//  contributors may be used to endorse or promote products derived from this
//  software without specific prior written permission.
//
//     THIS SOFTWARE MAY NOT BE USED FOR DIRECT COMMERCIAL ADVANTAGE WITHOUT
//     SPECIFIC PRIOR WRITTEN PERMISSION.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "PureMerge.H"
#include "Synchronous.H"
#include "Signal.H"
#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>

PureMerge::PureMerge(const char* name, const int nbEntries,
                     const boolean schedule)
          : Synchronous(name, schedule),
            fOutput(this, "output"),
            nbEntries(nbEntries)
{
  ostrstream *buf;
  fInputs = new (PureInSignal*[nbEntries + 1]);
  for(int i = 0; i < nbEntries; i++) {
    buf = new ostrstream;
    (*buf) << "iSig" << i;
    fInputs[i] = new PureInSignal(this,buf->str());
    delete buf;
  }
  fInputs[nbEntries] = 0;
}

PureMerge::~PureMerge() {
    for(int i = 0; i < nbEntries; i++)
      delete fInputs[i];
    delete[] fInputs;
}

void PureMerge::resetOutputs() {
  fOutput.value().reset();
}

void PureMerge::setInputs() {
  for(int i=0; i < nbEntries; i++)
    fInputs[i]->update();
}

void PureMerge::activate() {
  for(int i=0; i < nbEntries; i++) {
    if (fInputs[i]->value().isPresent()) {
      fOutput.value() = true;
      break;
    }
  }
}

const InputSignal** PureMerge::inSigList() const {
  const InputSignal** inList;
  
  inList = new (const InputSignal*[nbEntries + 1]);
  for(int i=0; i < (nbEntries + 1); i++)
    inList[i] = fInputs[i];
  return inList;
}

const OutputSignal** PureMerge::outSigList() const {
  const OutputSignal** outList;
  
  outList = new (const OutputSignal*[2]);
  outList[0] = &fOutput;
  outList[1] = 0;
  return outList;
}

int PureMerge::nbInputs() {
    return nbEntries;
}

PureInSignal& PureMerge::input(const int i) {
  if (i < nbEntries)
    return *(fInputs[i]);
  else {
    cerr << "# Index of input signal out of range" << endl;
    exit(1);
  }
}

PureOutSignal& PureMerge::output() {
  return fOutput;
}
