#ifndef _CLASS_XINTERFACE_H_
#define _CLASS_XINTERFACE_H_

//
//  File XInterface.H
//
//  July 1994 - September 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)


#include "Signal.H"
#include "IOModule.H"
#include "Array.H"
#include "SScheduler.H"

#define XTFUNCPROTO 1
#define NO_XMSTRINGS 1
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <strstream.h>

class XInterface {
	public:
		enum  { eValued, ePure, eLabel };

		XInterface(int argc, char **argv, const char *name,
			SScheduler *clock = 0);
		void logTo(ostream& logStream);
		void logTo(const char *logName);
		static void globalLogTo(ostream& logStream);
		static void globalLogTo(const char *logName);
		~XInterface();
		void showWindow();
		static void runsimul();

// The following members should be private, but g++ and cxx
// do not accept templates of friends, and even CC produces
// wrong C code when more than two template friends are
// declared with the same anonymous type!
		int fNbInputs;
		int fNbOutputs;
		SScheduler *fClock;
		Array<strstream> fInStreams;
		Array<strstream> fOutStreams;
		Array<char> fInNames;
		Array<char> fOutNames;
		Array<int> fInPure;
		Array<int> fOutPure;
		Array<Synchronous> fInModules;
// The following members should be private, but are used as Xt actions.
		static void resize(Widget w, XEvent *evt, String *parm, Cardinal *count);
	private:
		Array<struct _WidgetRec> fInToggle;
		Array<struct _WidgetRec> fInPush;
		Array<struct _WidgetRec> fInText;
		Array<struct _WidgetRec> fOutToggle;
		Array<struct _WidgetRec> fOutText;
		Widget fBad;
		Widget fScrollInp;
		Widget fScrollOutp;
		bool fMultiple;
		bool fSticky;
		bool fNamedLog;
		ostream *fLogStream;
		char *fName;
		
		static Widget fTopLevel;
		static XtAppContext fContext;
		static Array<XInterface> fInstances;
		static bool fMustQuit;
		static ostream *fGlobalLog;
		static bool fGlobalNamedLog;
		
		Widget fClockTop;
		void setupWindow();
		static void quitter(Widget w, XtPointer client, XtPointer call);
		static void reset(Widget w, XtPointer client, XtPointer call);
		static void cancelQuit(Widget w, XtPointer client, XtPointer call);
		static void reallyQuit(Widget w, XtPointer client, XtPointer call);
		static void tick(Widget w, XtPointer client, XtPointer call);
		static void multiple(Widget w, XtPointer client, XtPointer call);
		static void sticky(Widget w, XtPointer client, XtPointer call);
		static void input_text(Widget w, XtPointer client, XtPointer call);
		static void input_toggle(Widget w, XtPointer client, XtPointer call);
		static void input_push(Widget w, XtPointer client, XtPointer call);
		void doBad();
		void doTick();
		void adjustScrollWidth();
		XInterface(const XInterface&);
};

XInterface& plug(XInterface& x, PureInSignal& s);
XInterface& plug(XInterface& x, PureInSignal& s, const char* nickname);
XInterface& inputLabel(XInterface& x, const char *label);
XInterface& plug(XInterface& x, PureOutSignal& s);
XInterface& plug(XInterface& x, PureOutSignal& s, const char* nickname);
XInterface& outputLabel(XInterface& x, const char* label);

template<class sigType>
	XInterface& plug(XInterface& x, InSignal<sigType>& s);
template<class sigType>
	XInterface& plug(XInterface& x, InSignal<sigType>& s, const char* nickname);
template<class sigType>
	XInterface& plug(XInterface& x, OutSignal<sigType>& s);
template<class sigType>
	XInterface& plug(XInterface& x, OutSignal<sigType>& s, const char* nickname);

#ifdef __GNUG__
#include "XInterface.C"
#endif

#endif
