//
//  File coreProtoSim.C
//
//  September 1994 - October 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91192 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "ProtoSim.H"
#include <fstream.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>


bool ProtoSim::fMustQuit = false;
bool ProtoSim::fVerbose = true;
int ProtoSim::fMode = ProtoSim::eVerbose;
const char* ProtoSim::fVersion = "1.1";
Array<ProtoSim> ProtoSim::fInstances;

ProtoSim& plug(ProtoSim& x, PureInSignal& s) {
	ostrstream buf;
	buf << s.name() << ends;
	PureInModule *i = new PureInModule("","",true,buf.str(),false);
	strstream *t = new strstream;
	
	x.fInModules[x.fNbInputs] = i;
	x.fInStreams[x.fNbInputs] = t;
	i->setStreams(*t, nullDev, nullDev);
	x.fInNames[x.fNbInputs] = buf.str();
	x.fInPure[x.fNbInputs] = new int(ProtoSim::ePure);
	s << (*i)();
	x.fNbInputs++;
	return x;
}

ProtoSim& plug(ProtoSim& x, PureInSignal& s, const char *nickname) {
	ostrstream buf;
	buf << nickname << ends;
	PureInModule *i = new PureInModule("","",true,buf.str(),false);
	strstream *t = new strstream;
	
	x.fInModules[x.fNbInputs] = i;
	x.fInStreams[x.fNbInputs] = t;
	i->setStreams(*t, nullDev, nullDev);
	x.fInNames[x.fNbInputs] = buf.str();
	x.fInPure[x.fNbInputs] = new int(ProtoSim::ePure);
	s << (*i)();
	x.fNbInputs++;
	return x;
}

ProtoSim& plug(ProtoSim& x, PureOutSignal& s) {
	ostrstream buf;
	buf << s.name() << ends;
	PureOutModule *o = new PureOutModule("",true,"",buf.str(),false);
	x.fClock->manage(o);
	strstream *t = new strstream;
	
	x.fOutStreams[x.fNbOutputs] = t;
	o->setStreams(*t, nullDev);
	x.fOutNames[x.fNbOutputs] = buf.str();
	x.fOutPure[x.fNbOutputs] = new int(ProtoSim::ePure);
	(*o)() << s;
	x.fNbOutputs++;
	return x;
}

ProtoSim& plug(ProtoSim& x, PureOutSignal& s, const char *nickname) {
	ostrstream buf;
	buf << nickname << ends;
	PureOutModule *o = new PureOutModule("",true,"",buf.str(),false);
	x.fClock->manage(o);
	strstream *t = new strstream;
	
	x.fOutStreams[x.fNbOutputs] = t;
	o->setStreams(*t, nullDev);
	x.fOutNames[x.fNbOutputs] = buf.str();
	x.fOutPure[x.fNbOutputs] = new int(ProtoSim::ePure);
	(*o)() << s;
	x.fNbOutputs++;
	return x;
}

ProtoSim::ProtoSim(const char *name, SScheduler *clock)
			: fNbInputs(0),
			  fNbOutputs(0),
        fName(name),
			  fInstNum(fInstances.size()) {
	if (clock == 0)
		fClock = new SScheduler(name);
	theScheduler = fClock;
	fInstances[fInstances.size()] = this;
}

ProtoSim::~ProtoSim() {
}

ProtoSim* ProtoSim::lookUpInst(const char *name) {
	char *p;
	int n = fInstances.size();
	int i;
	
	if ((p = (char*)strrchr(name, '_')) == 0)  {
		if (isdigit(name[0]))
			i = atoi(name);
		else
			return 0;
	} else {
		*p = 0;
		i = atoi(p+1);
		if (strcmp(name, fInstances[i]->fName) != 0)
			return 0;
	}
	if (i < n)
		return fInstances[i];
	else
		return 0;
}

int ProtoSim::lookUpInput(const char *name) {
	char *p;
	
	if ((p = (char*)strrchr(name, '_')) == 0)  {
		if (isdigit(name[0])) {
			return atoi(name);
		} else
			return -1;
	} else {
		*p = 0;
		int n = atoi(p+1);
		if (strcmp(name, fInNames[n]) == 0)
			return n;
		else
			return -1;
	}
}

void ProtoSim::doInput(const char* name, const char* value) {
	int i = lookUpInput(name);

	if (i < 0) {
		if (fVerbose)
			cerr << "badName" << endl;
		else
			cerr << ":" << endl;
		return;
	}
	
	strstream *s = fInStreams[i];
	
	s->seekg(streampos(0));
	s->seekp(streampos(0));
	s->clear();
	if (value[0] != 0) {
		*s << value << endl << ends;
		fInModules[i]->react();
		if (!s->good()) {
			if (fVerbose)
				cerr << "valueRefused" << endl;
			else
				cerr << "/" << endl;
		} else {
			if (fVerbose)
				cerr << "valueAccepted" << endl;
			else
				cerr << "=" << endl;
		}
	} else {
		if (fVerbose)
			cerr << "valueRefused" << endl;
		else
			cerr << "/" << endl;
	}
}

void ProtoSim::doTick() {
	strstream *s;
	char c;
	int i;
	
	fClock->tick();
	
	for(i = 0; i < fNbInputs; i++) {	/* Reset input signals */
		fInModules[i]->resetOutputs();
	}
	
	for(i = 0; i < fNbOutputs; i++) {	/* Read and send output signals */
		ostrstream buf;
		char *v;

		s = fOutStreams[i];
		(*s) << ends;
		while((c = s->get()) != EOF)
			buf << c;
		buf << ends;
		s->seekg(streampos(0));
		s->seekp(streampos(0));
		s->clear();
		v = buf.str();
		if (v[0] != '#' || v[1] != '(') {
			if (fVerbose)
				cerr << "emitOutput " << fName << "_";
			else
				cerr << "! ";
			cerr << fInstNum << " ";
			if (fVerbose) 
				cerr << fOutNames[i] << "_";
			 cerr << i << " " << v << endl;
		}
		delete[] v;
	}
	if (fVerbose)
		cerr << "ready" << endl;
	else
		cerr << ">" << endl;
}

void ProtoSim::declare() {
  int i;
  
	fClock->backup();
	
	if (fVerbose)
		cerr << "clockName ";
	else
		cerr << "% ";
	cerr << fName << "_" << fInstNum << endl;
	
	for(i = 0; i < fNbInputs; i++) {
		if (*fInPure[i] == ePure)  {
			if (fVerbose)
				cerr << "declarePureInput " << fName << "_";
			else
				cerr << "+ ";
			cerr << fInstNum << " ";
			cerr << fInNames[i] << "_";
			cerr << i << endl;
		} else  {
			if (fVerbose)
				cerr << "declareInput " << fName << "_";
			else
				cerr << "i ";
			cerr << fInstNum << " ";
			cerr << fInNames[i] << "_" << i << " ";
			if (fInTypes[i] == 0)
				cerr << "?" << endl;
			else
				cerr << fInTypes[i] << endl;
		}
	}
	for(i = 0; i < fNbOutputs; i++) {
		if (*fOutPure[i] == ePure)  {
			if (fVerbose)
				cerr << "declarePureOutput " << fName << "_";
			else
				cerr << "* ";
			cerr << fInstNum << " ";
				cerr << fOutNames[i] << "_" << i << endl;
		} else  {
			if (fVerbose)
				cerr << "declareOutput " << fName << "_";
			else
				cerr << "o ";
			cerr << fInstNum << " ";
			cerr << fOutNames[i] << "_" << i << " ";
			if (fOutTypes[i] == 0)
				cerr << "?" << endl;
			else
				cerr << fOutTypes[i] << endl;
		}
	}
}

void ProtoSim::tick(istrstream *line) {
	char cname[256];
	
	*line >> cname;
	ProtoSim *p = lookUpInst(cname);
	if (p == 0)  {
		if (fVerbose)
			cerr << "badClock" << endl;
		else
			cerr << ";" << endl;
	} else
		p->doTick();
}

void ProtoSim::input(istrstream *line) {
	char cname[256];
	strstreambuf buf;
	
	*line >> cname;
	ProtoSim *p = lookUpInst(cname);
	if (p == 0) {
		if (fVerbose)
			cerr << "badClock" << endl;
		else
			cerr << ";" << endl;
	} else  {
		*line >> cname;
		line->get(buf);
		buf.sputc(0);
		p->doInput(cname,buf.str());
	}
}

void ProtoSim::reset(istrstream *line) {
	char cname[256];
	
	*line >> cname;
	ProtoSim *p = lookUpInst(cname);
	if (p == 0) {
		if (fVerbose)
			cerr << "badClock" << endl;
		else
			cerr << ";" << endl;
	} else
		p->doReset();
}

void ProtoSim::doReset() {
	fClock->restore();
	if (fVerbose)
		cerr << "ready" << endl;
	else
		cerr << ">" << endl;
}

void ProtoSim::runsimul() {
	char buf[256];
	char cmd[256];
	int n = fInstances.size();
	bool ok = false;
	
	
	cerr << "protosim " << fVersion << endl;
	
	while(!ok) {
		cin.getline(buf,255);
		if (strcmp(buf,"verbose") == 0) {
			fVerbose = true;
			fMode = eVerbose;
			ok = true;
		} else if (strcmp(buf,"-") == 0) {
			fVerbose = false;
			fMode = eConcise;
			ok = true;
		} else {
			if (fVerbose)
				cerr << "badCommand" << endl;
			else
				cerr << "\"" << endl;
		}
	}
	
	for(int i = 0; i < n; i++) {
		fInstances[i]->declare();
	}
	if (fVerbose)
		cerr << "ready" << endl;
	else
		cerr << ">" << endl;
	
	while (!fMustQuit) {
		cin.getline(buf,255);
		istrstream line(buf);
		
		line >> cmd;
		
		if ((strcmp(cmd,"?") == 0) || (strcmp(cmd,"setInput") == 0))
			input(&line);
		else if ((strcmp(cmd,"'") == 0) || (strcmp(cmd,"tick") == 0))
			tick(&line);
		else if ((strcmp(cmd,"<") == 0) || (strcmp(cmd,"reset") == 0))
			reset(&line);
		else if ((strcmp(cmd,".") == 0) || (strcmp(cmd,"stopSim") == 0))
			fMustQuit = true;
		else {
			if (fVerbose)
				cerr << "badCommand" << endl;
			else
				cerr << "\"" << endl;
		}
	}
}

