//
//  File Sample.C
//
//  September 3 1992 - August 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91190 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "Sample.H"
#include <ctype.h>
#include <string.h>

PureSample::PureSample() {
  fPresent = false;
}

PureSample::PureSample(const SBoolean& b) {
  fPresent = b;
}

PureSample::PureSample(const PureSample& s) {
  fPresent = s.fPresent;
}

bool PureSample::isPresent() const {
  return bool(fPresent);
}

PureSample& PureSample::operator=(const SBoolean&) {
  fPresent = true;
  return *this;
}

PureSample& PureSample::operator=(const PureSample& s) {
  fPresent = s.fPresent;
  return *this;
}

void PureSample::reset() {
  fPresent = false;
}

static const char str_true[] = "true";
static const char str_pres[] = "present";
static const char str_false[] = "false";
static const char str_notpres[] = "not present";

istream& operator>>(istream& i, PureSample& s) {
  char c;
  unsigned j = 0;
  const char* str = 0;
  bool done = false;
  bool fake = false;
  SBoolean val = false;

  if (!i.good())
    return i;
  
  if ((i.peek()) == '\n')
    return i;
  
  if (i.flags() & ios::skipws)
    i >> ws;
  
  if (((c = i.get()) == '#') && (i.peek() == '(')) {
    i.get();
    fake = true;
  } else
    i.putback(c);
  
  if (i.good()) {
    if (i.flags() & ios::skipws)
      i >> ws;
    while(int(!done) && i.get(c)) {
      if (str == 0) {
        switch(tolower(c)) {
         case '+':
           val = true;
           done = true;
           break;
         case '-':
           val = false;
           done = true;
           break;
         case 't':
           str = str_true;
           j++;
           break;
         case 'p':
           str = str_pres;
           j++;
           break;
         case 'f':
           str = str_false;
           j++;
           break;
         case 'n':
           str = str_notpres;
           j++;
           break;
         default:
           i.putback(c);
           i.clear(i.rdstate() | ios::failbit);
           break;
        }
      } else {
        if (tolower(c) != str[j++]) {
          i.putback(c);
          i.clear(i.rdstate() | ios::badbit);
        } else {
          if (j >= strlen(str)) {
            if ((str == str_true) || (str == str_pres))
              val = true;
            else
              val = false;
            done = true;
          }
        }
      }
    }
  }

  if (i.good() && !fake) {
      s.fPresent = val;
  }
  if (fake) {
    if (i.flags() & ios::skipws)  // If should skip white spaces
      i >> ws;
    if (i.peek() == ')')
      i.get();
    else
      i.clear(i.rdstate() | ios::badbit);
  }
  return i;
}

ostream& operator<<(ostream& o, const PureSample& s) {
  if (s.fPresent)
    return o << "present";
  else
    return o << "#(not present)";
}

// End of file coreSample.C
