//
//  File coreXInterface.C
//
//  July 1994 - September 1994
//
//  Frederic Boulanger - boulange@supelec.fr
//  Ecole Superieure d Electricite - Service Informatique
//  Plateau de Moulon - 91192 Gif sur Yvette cedex
//  France
//
//  This file is part of the libSync synchronous library
//  Copyright Supelec, 1992-1994.
//  All Rights Reserved.
//  
//  Redistribution and use are permitted provided that:
//    - distributions including source files retain this entire copyright
//      notice and comment,
//    - distributions including binaries display the following acknolegment :
//      "This product includes software developped by Supelec
//       and its contributors" in the documentation or other
//      materials provided with the distribution and in all advertising materials
//      mentionning features or use of this software.
//
//     THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
//     WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
//     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
//
//  Author: Frederic Boulanger (Frederic.Boulanger@supelec.fr)

#include "XInterface.H"
#include <Xm/Label.h>
#include <Xm/Form.h>
#include <Xm/MessageB.h>
#include <Xm/TextF.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/Separator.h>
#include <Xm/ScrolledW.h>
#include <Xm/PanedW.h>
#include <Xm/MwmUtil.h>
#include <X11/keysym.h>
#include <fstream.h>
#include <stdlib.h>

bool XInterface::fMustQuit = false;
Widget XInterface::fTopLevel;
XtAppContext XInterface::fContext;
Array<XInterface> XInterface::fInstances;
ostream *XInterface::fGlobalLog = 0;
bool XInterface::fGlobalNamedLog = false;

static XtActionsRec actionTable[] = {
	{ "Resize", XInterface::resize}
};

static String scrollTransTable = "<Configure>:Resize()";

static XtTranslations theScrollTranslation;

XInterface& plug(XInterface& x, PureInSignal& s) {
	ostrstream buf;
	buf << s.name() << ends;
	PureInModule *i = new PureInModule("","",true,buf.str(),false);
	strstream *t = new strstream;
	
	x.fInModules[x.fNbInputs] = i;
	x.fInStreams[x.fNbInputs] = t;
	i->setStreams(*t, nullDev, nullDev);
	x.fInNames[x.fNbInputs] = buf.str();
	x.fInPure[x.fNbInputs] = new int(XInterface::ePure);
	s << (*i)();
	x.fNbInputs++;
	return x;
}

XInterface& plug(XInterface& x, PureInSignal& s, const char *nickname) {
	ostrstream buf;
	buf << nickname << ends;
	PureInModule *i = new PureInModule("","",true,buf.str(),false);
	strstream *t = new strstream;
	
	x.fInModules[x.fNbInputs] = i;
	x.fInStreams[x.fNbInputs] = t;
	i->setStreams(*t, nullDev, nullDev);
	x.fInNames[x.fNbInputs] = buf.str();
	x.fInPure[x.fNbInputs] = new int(XInterface::ePure);
	s << (*i)();
	x.fNbInputs++;
	return x;
}

XInterface& inputLabel(XInterface& x, const char *label) {
	ostrstream buf;
	buf << label << ends;
	
	x.fInModules[x.fNbInputs] = 0;
	x.fInStreams[x.fNbInputs] = 0;
	x.fInNames[x.fNbInputs] = buf.str();
	x.fInPure[x.fNbInputs] = new int(XInterface::eLabel);
	x.fNbInputs++;
	return x;
}

XInterface& plug(XInterface& x, PureOutSignal& s) {
	ostrstream buf;
	buf << s.name() << ends;
	PureOutModule *o = new PureOutModule("",true,"",buf.str(),false);
	x.fClock->manage(o);
	strstream *t = new strstream;
	
	x.fOutStreams[x.fNbOutputs] = t;
	o->setStreams(*t, nullDev);
	x.fOutNames[x.fNbOutputs] = buf.str();
	x.fOutPure[x.fNbOutputs] = new int(XInterface::ePure);
	(*o)() << s;
	x.fNbOutputs++;
	return x;
}

XInterface& plug(XInterface& x, PureOutSignal& s, const char* nickname) {
	ostrstream buf;
	buf << nickname << ends;
	PureOutModule *o = new PureOutModule("",true,"",buf.str(),false);
	x.fClock->manage(o);
	strstream *t = new strstream;
	
	x.fOutStreams[x.fNbOutputs] = t;
	o->setStreams(*t, nullDev);
	x.fOutNames[x.fNbOutputs] = buf.str();
	x.fOutPure[x.fNbOutputs] = new int(XInterface::ePure);
	(*o)() << s;
	x.fNbOutputs++;
	return x;
}

XInterface& outputLabel(XInterface& x, const char* label) {
	ostrstream buf;
	buf << label << ends;
	
	x.fOutStreams[x.fNbOutputs] = 0;
	x.fOutNames[x.fNbOutputs] = buf.str();
	x.fOutPure[x.fNbOutputs] = new int(XInterface::eLabel);
	x.fNbOutputs++;
	return x;
}

XInterface::XInterface(int argc, char **argv, const char *name, SScheduler *clock)
			: fNbInputs(0),
			  fNbOutputs(0),
			  fBad(0),
			  fMultiple(true),
			  fSticky(false),
			  fNamedLog(false),
			  fLogStream(0) {	
	if (clock == 0)
		fClock = new SScheduler(name);
	theScheduler = fClock;
	if (fInstances.size() == 0) {
#ifdef MOTIF_1_1
		fTopLevel = XtAppInitialize(&fContext,"SyncXInterface",NULL,0,
								(Cardinal*)&argc,argv,NULL,0,0);
#else
		fTopLevel = XtAppInitialize(&fContext,"SyncXInterface",NULL,0,
								&argc,argv,NULL,0,0);
#endif
		XtAppAddActions(fContext,actionTable,sizeof(actionTable)/sizeof(XtActionsRec));
		theScrollTranslation = XtParseTranslationTable(scrollTransTable);
	}
	fInstances[fInstances.size()] = this;
	ostrstream s;
	s << argv[0] << "::" << name << ends;
	fName = s.str();
	Arg arg[1];
	XtSetArg(arg[0], XmNtitle, fName);
	fClockTop = XtCreatePopupShell(fName,topLevelShellWidgetClass,fTopLevel,arg,1);
}

void XInterface::showWindow() {
	setupWindow();
	fClock->backup();
	XtPopup(fClockTop, XtGrabNone);
	adjustScrollWidth();
}

void XInterface::logTo(ostream& log) {
	fLogStream = &log;
	*fLogStream << "--- " << fName << endl;
}

void XInterface::logTo(const char *logName) {
	fLogStream = new ofstream(logName);
	fNamedLog = true;
	*fLogStream << "--- " << fName << endl;
}

void XInterface::globalLogTo(ostream& log) {
	fGlobalLog = &log;
}

void XInterface::globalLogTo(const char *logName) {
	fGlobalLog = new ofstream(logName);
	fGlobalNamedLog = true;
}

XInterface::~XInterface() {
	if (fNamedLog)
		((ofstream*)fLogStream)->close();
}

void XInterface::resize(Widget w, XEvent*, String*, Cardinal*) {
	Widget theForm, theClip;
	Dimension width;
	Arg arg[3];
	
	XtSetArg(arg[0], XmNworkWindow, &theForm);
	XtSetArg(arg[1], XmNclipWindow, &theClip);
	XtGetValues(w, arg, 2);
	XtSetArg(arg[0], XmNwidth, &width);
	XtGetValues(theClip, arg,1);
	width -= 10;
	XtSetArg(arg[0], XmNwidth, width);
	XtSetValues(theForm, arg,1);
}

void XInterface::doBad() {
	XmProcessTraversal(fBad,XmTRAVERSE_CURRENT);
}

void XInterface::cancelQuit(Widget, XtPointer dlog, XtPointer) {
	Widget w = Widget(dlog);
	XtPopdown(w);
	XtDestroyWidget(w);
}

void XInterface::reallyQuit(Widget, XtPointer dlog, XtPointer) {
	Widget w = Widget(dlog);
	XtPopdown(w);
	XtDestroyWidget(w);
	fMustQuit = true;
}

void XInterface::quitter(Widget w,XtPointer, XtPointer) {
	Arg arg[5];
	int i = 0;
	Widget confShell;
	Screen *scr;

	XtSetArg(arg[i], XmNdialogTitle,
		XmStringCreateLtoR("Confirm quit",XmSTRING_DEFAULT_CHARSET));i++;
	XtSetArg(arg[i], XmNdialogStyle,XmDIALOG_FULL_APPLICATION_MODAL); i++;
	XtSetArg(arg[i], XmNmwmDecorations,
		MWM_DECOR_ALL|MWM_DECOR_RESIZEH|MWM_DECOR_TITLE|MWM_DECOR_MENU|MWM_DECOR_MINIMIZE|MWM_DECOR_MAXIMIZE); i++;
	confShell = XtCreatePopupShell("ConfirmQuit",transientShellWidgetClass,fTopLevel,arg,i);

	i = 0;
	XtSetArg(arg[i], XmNdialogType, XmDIALOG_QUESTION); i++;
	XtSetArg(arg[i], XmNmessageString,
		XmStringCreateLtoR("Really quit?",XmSTRING_DEFAULT_CHARSET));i++;

	Widget dlog = XmCreateMessageBox(confShell,"ConfirmMessage",arg,i);

	XtDestroyWidget(XmMessageBoxGetChild(dlog, XmDIALOG_HELP_BUTTON));
	XtAddCallback(dlog,XmNcancelCallback,cancelQuit,XtPointer(confShell));
	XtAddCallback(dlog,XmNokCallback,reallyQuit,XtPointer(confShell));
	XtManageChild(dlog);
	
	scr = XtScreen(w);
	XtSetArg(arg[0], XmNx, WidthOfScreen(scr)/3);
	XtSetArg(arg[1], XmNy, HeightOfScreen(scr)/3);
	XtSetValues(confShell,arg,2);
	
	XtPopup(confShell, XtGrabExclusive);
}

void XInterface::reset(Widget,XtPointer xi, XtPointer) {
	XInterface *x = (XInterface*)xi;
	
	if (x->fBad)
		x->doBad();
	else {
		Arg arg[1];
		x->fClock->restore();
		for(int i = 0; i < x->fNbOutputs; i++) {
			if (*(x->fOutPure[i]) == eLabel)
				continue;
			XtSetArg(arg[0],XmNvalue,"");
			XtSetValues(x->fOutText[i],arg,1);
			XtSetArg(arg[0],XmNset,False);
			XtSetValues(x->fOutToggle[i],arg,1);
		}
		if (x->fLogStream)
		  (*(x->fLogStream)) << "--- reset" << endl;
		if (fGlobalLog)
		  (*fGlobalLog) << "--- " << x->fName << " : reset" << endl;
	}
}

void XInterface::tick(Widget,XtPointer xi, XtPointer) {
	XInterface *x = (XInterface*)xi;
	
	if (x->fBad)
		x->doBad();
	else
		x->doTick();
}

void XInterface::multiple(Widget, XtPointer xi, XtPointer) {
	XInterface *x = (XInterface*)xi;
	
	x->fMultiple = !x->fMultiple;
	if (x->fNbInputs == 0)
		return;
	for(int i = 0; i < x->fNbInputs; i++) {
		if (*(x->fInPure[i]) == eLabel)
			continue;
		XtSetMappedWhenManaged(x->fInPush[i], !x->fMultiple);
		XtSetMappedWhenManaged(x->fInToggle[i], x->fMultiple);
	}
	if (! x->fMultiple) {
		Arg arg[1];
		XtSetArg(arg[0], XmNset, False);
		for(int i = 0; i < x->fNbInputs; i++) {
			if (*(x->fInPure[i]) == eLabel)
				continue;
			XtSetValues(x->fInToggle[i], arg, 1);
		}
	}
	if (x->fBad != 0)
		x->doBad();
}

void XInterface::sticky(Widget, XtPointer xi, XtPointer) {
	XInterface *x = (XInterface*)xi;
	
	x->fSticky = !x->fSticky;
	if (x->fBad != 0)
		x->doBad();
}

void XInterface::input_text(Widget w, XtPointer idx, XtPointer verif) {
	Arg arg[1];
	XInterface *x;
	
	XtSetArg(arg[0], XmNuserData, &x);
	XtGetValues(w,arg,1);
	int i = int(idx);
	XmTextVerifyPtr textverif = XmTextVerifyPtr(verif);
	strstream *s = x->fInStreams[i];
	char *v;
	
	if (
			(textverif->reason == XmCR_MODIFYING_TEXT_VALUE)
		||
			(textverif->reason == XmCR_MOVING_INSERT_CURSOR)
	) {
		if ((x->fBad != 0) && (x->fBad != w)) {
			textverif->doit = False;
			x->doBad();
		}
		return;
	}
	
//	losing focus: check value.

	s->seekg(streampos(0));
	s->seekp(streampos(0));
	s->clear();
	XtSetArg(arg[0], XmNvalue, &v);
	XtGetValues(w,arg,1);
	if (v[0] != 0) {
		*s << v << endl << ends;
		x->fInModules[i]->react();
		if (!s->good()) {
			textverif->doit = False;
			x->fBad = w;
			XmTextFieldSetSelection(w,
				0,XmTextFieldGetLastPosition(w),
				textverif->event->xselection.time);
		} else {
			if (x->fBad == w)
				x->fBad = 0;
		}
		s->seekg(streampos(0));
		s->seekp(streampos(0));
		s->clear();
	} else
		if (x->fBad == w)
			x->fBad = 0;
}

void XInterface::input_toggle(Widget w, XtPointer, XtPointer) {
	XInterface *x;
	Arg arg[1];
	
	XtSetArg(arg[0],XmNuserData,&x);
	XtGetValues(w,arg,1);

	if (x->fBad != 0)
		x->doBad();
}

void XInterface::input_push(Widget w, XtPointer idx, XtPointer) {
	Arg arg[1];
	XInterface *x;
	int i = int(idx);
	
	XtSetArg(arg[0],XmNuserData,&x);
	XtGetValues(w,arg,1);
	
	if (x->fBad) {
		x->doBad();
		return;
	}
	XtSetArg(arg[0], XmNset, True);
	XtSetValues(x->fInToggle[i], arg,1);
	x->doTick();
}

void XInterface::doTick() {
	strstream *s;
	Synchronous *inp;
	char *v;
	char c;
	int i;
	Boolean isSet;
	Arg arg[1];
	
	if (fBad) {
		doBad();
		return;
	}
	
	if(fLogStream)
		(*fLogStream) << "-> ";
	if(fGlobalLog)
		(*fGlobalLog) << "--- " << fName << endl << "-> ";
	
	for(i = 0; i < fNbInputs; i++) {
		if (*fInPure[i] == eLabel)
			continue;
		s = fInStreams[i];
		inp = fInModules[i];
		s->seekg(streampos(0));
		s->seekp(streampos(0));
		s->clear();
		XtSetArg(arg[0],XmNset,&isSet);
		XtGetValues(fInToggle[i],arg,1);
		if (isSet) {
			if (*fInPure[i]) {
				*s << "present" << endl;
				if(fLogStream)
					(*fLogStream) << fInNames[i] << "; ";
				if(fGlobalLog)
					(*fGlobalLog) << fInNames[i] << "; ";
			} else {
				XtSetArg(arg[0],XmNvalue,&v);
				XtGetValues(fInText[i],arg,1);
				if (v[0] == 0)
					*s << endl;
				else {
					*s << v << endl;
					if(fLogStream)
						(*fLogStream) << fInNames[i] << "(" << v << "); ";
					if(fGlobalLog)
						(*fGlobalLog) << fInNames[i] << "(" << v << "); ";
				}
				XtFree(v);
			}
			if(!fSticky || !fMultiple) {
				XtSetArg(arg[0],XmNset,False);
				XtSetValues(fInToggle[i],arg,1);
			}
		} else {
			*s << endl;
		}
		*s << ends;
		inp->resetOutputs();
		inp->react();
	}
	if(fLogStream)
		(*fLogStream) << endl << "<- ";
	if(fGlobalLog)
		(*fGlobalLog) << endl << "<- ";

	fClock->tick();
	
	for(i = 0; i < fNbOutputs; i++) {
		ostrstream buf;
		if (*fOutPure[i] == eLabel)
			continue;
		s = fOutStreams[i];
		(*s) << ends;
		while((c = s->get()) != EOF)
			buf << c;
		buf << ends;
		s->seekg(streampos(0));
		s->seekp(streampos(0));
		s->clear();
		v = buf.str();
		XtSetArg(arg[0],XmNvalue,v);
		XtSetValues(fOutText[i],arg,1);
		if (v[0] == '#' && v[1] == '(')
			XtSetArg(arg[0],XmNset,False);
		else {
			XtSetArg(arg[0],XmNset,True);
			if(fLogStream) {
				if (*fOutPure[i])
					(*fLogStream) << fOutNames[i] << "; ";
				else
					(*fLogStream) << fOutNames[i] << "(" << v << "); ";
			}
			if(fGlobalLog) {
				if (*fOutPure[i])
					(*fGlobalLog) << fOutNames[i] << "; ";
				else
					(*fGlobalLog) << fOutNames[i] << "(" << v << "); ";
			}
		}
		XtSetValues(fOutToggle[i],arg,1);
		delete[] v;
	}
	if(fLogStream)
		(*fLogStream) << endl << endl << flush;
	if(fGlobalLog)
		(*fGlobalLog) << endl << endl << flush;
}

void XInterface::adjustScrollWidth() {
	resize(fScrollInp,0,0,0);
	resize(fScrollOutp,0,0,0);
}

void XInterface::setupWindow() {
	Arg arg[15];
	int i;
	Widget contMain, versLabel, contCtrl,paneW, sv,sh;
	Widget workInp, workOutp;
	Widget form, prevform = 0;
	Array<struct _WidgetRec> outLabel;
	Widget btick,cmult,cstick,breset,bquit,dummy;
	Dimension maxWidth = 0, curWidth;
	Dimension maxHeight = 0, curHeight;
	XFontStruct *versFont;
	XmFontList fontList;
	ostrstream buf;
	int j;
	
	buf << "SyncXInterface v2.1, " << libVersion() << ends;
	
	contMain = XmCreateForm(fClockTop,"MainForm",NULL,0);
	XtManageChild(contMain);
	
	versFont = XLoadQueryFont(XtDisplay(fClockTop),
		"-*-helvetica-medium-r-*-*-10-*-*-*-*-*-iso8859-*");
	fontList = XmFontListCreate(versFont,"iso8859");
	
	i = 0;
	XtSetArg(arg[i], XmNfontList, fontList);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNlabelString,
		XmStringCreateLtoR(buf.str(), XmSTRING_DEFAULT_CHARSET));i++;
	versLabel = XmCreateLabel(contMain,"Version",arg,i);
	XtManageChild(versLabel);
	XmFontListFree(fontList);
	delete[] buf.str();
	
	i = 0;
	XtSetArg(arg[i], XmNorientation, XmHORIZONTAL);i++;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNtopOffset, 5);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNtopWidget, versLabel);i++;
	sh = XmCreateSeparator(contMain,"VersSep",arg,i);
	XtManageChild(sh);
	
	i = 0;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNtopWidget, sh);i++;
	XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
	contCtrl = XmCreateForm(contMain,"ControlsForm",arg,i);
	XtManageChild(contCtrl);
	
	i = 0;
	XtSetArg(arg[i], XmNorientation, XmVERTICAL);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNtopWidget, sh);i++;
	XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightOffset, 5);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNrightWidget, contCtrl);i++;
	sv = XmCreateSeparator(contMain,"VertSep",arg,i);
	XtManageChild(sv);
	
	i = 0;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNrightOffset, 5);i++;
	XtSetArg(arg[i], XmNrightWidget, sv);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNtopWidget, sh);i++;
	XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
	paneW = XmCreatePanedWindow(contMain,"SigPane", arg,i);
	XtManageChild(paneW);

	i = 0;
	XtSetArg(arg[i], XmNscrollBarDisplayPolicy, XmAS_NEEDED); i++;
	XtSetArg(arg[i], XmNscrollingPolicy, XmAUTOMATIC); i++;
	XtSetArg(arg[i], XmNvisualPolicy, XmCONSTANT); i++;
	XtSetArg(arg[i], XmNallowResize, True); i++;
	fScrollInp = XmCreateScrolledWindow(paneW,"ScrollInp", arg, i);
	XtManageChild(fScrollInp);
	XtAugmentTranslations(fScrollInp, theScrollTranslation);

	i = 0;
	workInp = XmCreateForm(fScrollInp,"WorkInp",arg,i);
	XtManageChild(workInp);
	XtSetArg(arg[0], XmNworkWindow, workInp);
	XtSetValues(fScrollInp, arg, 1);

	i = 0;
	XtSetArg(arg[i], XmNscrollBarDisplayPolicy, XmAS_NEEDED); i++;
	XtSetArg(arg[i], XmNscrollingPolicy, XmAUTOMATIC); i++;
	XtSetArg(arg[i], XmNvisualPolicy, XmCONSTANT); i++;
	XtSetArg(arg[i], XmNallowResize, True); i++;
	fScrollOutp = XmCreateScrolledWindow(paneW,"ScrollOutp", arg, i);
	XtManageChild(fScrollOutp);
	XtAugmentTranslations(fScrollOutp, theScrollTranslation);

	i = 0;
	workOutp = XmCreateForm(fScrollOutp,"WorkOutp",arg,i);
	XtManageChild(workOutp);
	XtSetArg(arg[0], XmNworkWindow, workOutp);
	XtSetValues(fScrollOutp, arg, 1);
	
	i = 0;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR("Tick",XmSTRING_DEFAULT_CHARSET));i++;
	btick = XmCreatePushButton(contCtrl,"tick",arg,i);
	XtAddCallback(btick,XmNactivateCallback,tick,(XtPointer)this);
	XtManageChild(btick);

	i = 0;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNtopWidget, btick);i++;
	XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR("Multi",XmSTRING_DEFAULT_CHARSET));i++;
	XtSetArg(arg[i], XmNindicatorType, XmN_OF_MANY);i++;
	XtSetArg(arg[i], XmNset, True);i++;
	cmult = XmCreateToggleButton(contCtrl,"multi",arg,i);
	XtAddCallback(cmult,XmNvalueChangedCallback,multiple,(XtPointer)this);
	XtManageChild(cmult);
	
	i = 0;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNtopWidget, cmult);i++;
	XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR("Sticky",XmSTRING_DEFAULT_CHARSET));i++;
	XtSetArg(arg[i], XmNindicatorType, XmN_OF_MANY);i++;
	cstick = XmCreateToggleButton(contCtrl,"sticky",arg,i);
	XtAddCallback(cstick,XmNvalueChangedCallback,sticky,(XtPointer)this);
	XtManageChild(cstick);
	
	i = 0;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR("Quit",XmSTRING_DEFAULT_CHARSET));i++;
	bquit = XmCreatePushButton(contCtrl,"quit",arg,i);
	XtAddCallback(bquit,XmNactivateCallback,quitter,(XtPointer)this);
	XtManageChild(bquit);
	
	i = 0;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNbottomWidget, bquit);i++;
	XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR("Reset",XmSTRING_DEFAULT_CHARSET));i++;
	breset = XmCreatePushButton(contCtrl,"breset",arg,i);
	XtAddCallback(breset,XmNactivateCallback,reset,(XtPointer)this);
	XtManageChild(breset);
	
	i = 0;
	XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
	XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNtopWidget, cstick);i++;
	XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_WIDGET);i++;
	XtSetArg(arg[i], XmNbottomWidget, breset);i++;
	dummy = XmCreateForm(contCtrl,"dummy",arg,i);
	XtManageChild(dummy);

	for(j = 0; j < fNbInputs; j++) {
		i = 0;
		if (j == 0) {
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
		} else {
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
			XtSetArg(arg[i], XmNtopWidget, prevform);i++;
		}
		XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
		XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
		form = XmCreateForm(workInp, "",arg,i);
		XtManageChild(form);
		prevform = form;
		
		if (*fInPure[j] != eLabel) {
			i = 0;
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR(fInNames[j],XmSTRING_DEFAULT_CHARSET));i++;
			XtSetArg(arg[i], XmNuserData, (XtPointer)this);i++;
			XtSetArg(arg[i], XmNalignment, XmALIGNMENT_BEGINNING);i++;
			fInToggle[j] = XmCreateToggleButton(form,"",arg,i);
			XtManageChild(fInToggle[j]);
			XtSetArg(arg[i-1], XmNalignment, XmALIGNMENT_CENTER);
			fInPush[j] = XmCreatePushButton(form,"",arg,i);
			XtManageChild(fInPush[j]);
			XtAddCallback(fInToggle[j],XmNvalueChangedCallback,input_toggle,0);
			XtAddCallback(fInPush[j],XmNactivateCallback,input_push,XtPointer(j));
	
			i = 0;
			XtSetArg(arg[i], XmNorientation, XmVERTICAL);i++;
			XtSetArg(arg[i], XmNseparatorType, XmNO_LINE);i++;
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNleftAttachment, XmATTACH_WIDGET);i++;
			XtSetArg(arg[i], XmNleftWidget, fInToggle[j]);i++;
			XtSetArg(arg[i], XmNleftOffset, 5);i++;
			sv = XmCreateSeparator(form,"",arg,i);
			XtManageChild(sv);
	
			i = 0;
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNvalue, "");i++;
			XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNleftAttachment, XmATTACH_WIDGET);i++;
			XtSetArg(arg[i], XmNleftWidget, sv);i++;
			XtSetArg(arg[i], XmNleftOffset, 5);i++;
			XtSetArg(arg[i], XmNuserData, (XtPointer)this);i++;
			fInText[j] = XmCreateTextField(form,"",arg,i);
			XtAddCallback(fInText[j],XmNlosingFocusCallback,input_text,XtPointer(j));
			XtAddCallback(fInText[j],XmNmotionVerifyCallback,input_text,XtPointer(j));
			XtAddCallback(fInText[j],XmNmodifyVerifyCallback,input_text,XtPointer(j));
			if(!(*fInPure[j]))
				XtManageChild(fInText[j]);
		} else  {
			i = 0;
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR(fInNames[j], XmSTRING_DEFAULT_CHARSET));i++;
			fInPush[j] = XmCreateLabel(form,"",arg,i);
			XtManageChild(fInPush[j]);
		}
	}
	for(j = 0; j < fNbOutputs; j++) {
		i = 0;
		if (j == 0) {
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
		} else {
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_WIDGET);i++;
			XtSetArg(arg[i], XmNtopWidget, prevform);i++;
		}
		XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
		XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
		form = XmCreateForm(workOutp,"",arg,i);
		prevform = form;
		XtManageChild(form);
		
		if (*fOutPure[j] != eLabel) {
			i = 0;
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNsensitive, False);i++;
			XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR("",XmSTRING_DEFAULT_CHARSET));i++;
			fOutToggle[j] = XmCreateToggleButton(form,"",arg,i);
			XtManageChild(fOutToggle[j]);
	
			i = 0;
			XtSetArg(arg[i], XmNalignment, XmALIGNMENT_BEGINNING);i++;
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNleftAttachment, XmATTACH_WIDGET);i++;
			XtSetArg(arg[i], XmNleftWidget, fOutToggle[j]);i++;
			XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR(fOutNames[j],XmSTRING_DEFAULT_CHARSET));i++;
			outLabel[j] = XmCreateLabel(form,"",arg,i);
			XtManageChild(outLabel[j]);
	
			i = 0;
			XtSetArg(arg[i], XmNsensitive, True);i++;
			XtSetArg(arg[i], XmNeditable, False);i++;
			XtSetArg(arg[i], XmNcursorPositionVisible, False);i++;
			XtSetArg(arg[i], XmNshadowThickness, 1);i++;
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNleftAttachment, XmATTACH_WIDGET);i++;
			XtSetArg(arg[i], XmNleftWidget, outLabel[j]);i++;
			XtSetArg(arg[i], XmNleftOffset, 10);i++;
			XtSetArg(arg[i], XmNuserData, (XtPointer)this);i++;
			fOutText[j] = XmCreateTextField(form,"",arg,i);
			if (!(*fOutPure[j]))
				XtManageChild(fOutText[j]);
		} else {
			i = 0;
			XtSetArg(arg[i], XmNtopAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNbottomAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNleftAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNrightAttachment, XmATTACH_FORM);i++;
			XtSetArg(arg[i], XmNlabelString, XmStringCreateLtoR(fOutNames[j], XmSTRING_DEFAULT_CHARSET));i++;
			fOutToggle[j] = XmCreateLabel(form,"",arg,i);
			XtManageChild(fOutToggle[j]);
		}
	}

	maxHeight = 0;
	maxWidth = 0;
	for(j = 0; j < fNbInputs; j++) {
		if (*fInPure[j] == eLabel)
			continue;
		XtSetArg(arg[0], XmNwidth, &curWidth);
		XtSetArg(arg[1], XmNheight, &curHeight);
		XtGetValues(fInToggle[j],arg,2);
		if (curWidth > maxWidth)
			maxWidth = curWidth;
		if (curHeight > maxHeight)
			maxHeight = curHeight;
	}
	if (maxWidth == 0)
		maxWidth = 100;
	if (maxHeight == 0)
		maxHeight = 20;
	for(j = 0; j < fNbInputs; j++) {
		if (*fInPure[j] == eLabel)
			continue;
		XtSetArg(arg[0], XmNwidth, maxWidth);
		XtSetValues(fInToggle[j],arg,1);
		XtSetValues(fInPush[j],arg,1);
	}
	XtSetArg(arg[0], XmNpaneMinimum, 3*maxHeight);
	XtSetValues(fScrollInp,arg,1);
	
	maxHeight = 0;
	maxWidth = 0;
	for(j = 0; j < fNbOutputs; j++) {
		if (*fOutPure[j] == eLabel)
			continue;
		XtSetArg(arg[0], XmNwidth, &curWidth);
		XtSetArg(arg[1], XmNheight, &curHeight);
		XtGetValues(outLabel[j],arg,2);
		if (curWidth > maxWidth)
			maxWidth = curWidth;
		if (curHeight > maxHeight)
			maxHeight = curHeight;
	}
	if (maxWidth == 0)
		maxWidth = 100;
	if (maxHeight == 0)
		maxHeight = 20;
	for(j = 0; j < fNbOutputs; j++) {
		if (*fOutPure[j] == eLabel)
			continue;
		XtSetArg(arg[0], XmNwidth, maxWidth);
		XtSetValues(outLabel[j],arg,1);
	}
	XtSetArg(arg[0], XmNpaneMinimum, 3*maxHeight);
	XtSetValues(fScrollOutp,arg,1);
	
	XtSetArg(arg[0], XmNwidth, &curWidth);
	XtGetValues(versLabel, arg, 1);
	XtSetArg(arg[0], XmNheight, &curHeight);
	XtGetValues(cmult, arg, 1);
	maxWidth = curWidth;
	maxHeight = 8 * curHeight;
	XtSetArg(arg[0],XmNminWidth, maxWidth);
	XtSetArg(arg[1],XmNminHeight, maxHeight);
	XtSetValues(fClockTop, arg,2);
}

void XInterface::runsimul() {
	XEvent evt;
	int n = fInstances.size();
	
	for(int i = 0; i < n; i++) {
		fInstances[i]->showWindow();
	}
	while (!fMustQuit) {
		XtAppNextEvent(fContext,&evt);
		XtDispatchEvent(&evt);
	}
	if (fGlobalLog && fGlobalNamedLog)
		((ofstream*)fGlobalLog)->close();
}

